Class: Textbringer::Buffer
- Inherits:
-
Object
- Object
- Textbringer::Buffer
- Extended by:
- Enumerable
- Defined in:
- lib/textbringer/buffer.rb
Defined Under Namespace
Modules: RectangleMethods
Constant Summary collapse
- GAP_SIZE =
256- UNDO_LIMIT =
1000- UTF8_CHAR_LEN =
Hash.new(1)
- DEFAULT_DETECT_ENCODING =
->(s) { @@auto_detect_encodings.find { |e| s.force_encoding(e) s.valid_encoding? } }
- NKF_DETECT_ENCODING =
->(s) { e = NKF.guess(s) case e when Encoding::US_ASCII if s.ascii_only? Encoding::UTF_8 else Encoding::ASCII_8BIT end when Encoding::ASCII_8BIT s.force_encoding(Encoding::UTF_8) if s.valid_encoding? Encoding::UTF_8 else s.force_encoding(Encoding::ASCII_8BIT) Encoding::ASCII_8BIT end else e end }
- BYTESPLICE_SUPPORTS_PARTIAL_COPY =
begin (+"foo").bytesplice(0, 2, "bar", 1, 2) == "aro" rescue NoMethodError, ArgumentError false end
- WORD_COMPONENT_REGEXP =
/\p{Letter}|\p{Number}|\p{M}/- @@auto_detect_encodings =
[ Encoding::UTF_8, Encoding::EUC_JP, Encoding::Windows_31J ]
- @@detect_encoding_proc =
DEFAULT_DETECT_ENCODING- @@table =
{}
- @@list =
[]
- @@current =
nil- @@minibuffer =
nil- @@global_mark_ring =
nil
Instance Attribute Summary collapse
-
#current_column ⇒ Object
readonly
Returns the value of attribute current_column.
-
#current_line ⇒ Object
readonly
Returns the value of attribute current_line.
-
#file_encoding ⇒ Object
Returns the value of attribute file_encoding.
-
#file_format ⇒ Object
Returns the value of attribute file_format.
-
#file_name ⇒ Object
Returns the value of attribute file_name.
-
#input_method ⇒ Object
readonly
Returns the value of attribute input_method.
-
#isearch_mark ⇒ Object
readonly
Returns the value of attribute isearch_mark.
-
#keymap ⇒ Object
Returns the value of attribute keymap.
-
#last_match ⇒ Object
readonly
Returns the value of attribute last_match.
-
#mark_active ⇒ Object
readonly
Returns the value of attribute mark_active.
-
#marks ⇒ Object
readonly
Returns the value of attribute marks.
-
#mode ⇒ Object
Returns the value of attribute mode.
-
#name ⇒ Object
Returns the value of attribute name.
-
#point ⇒ Object
readonly
Returns the value of attribute point.
-
#visible_mark ⇒ Object
readonly
Returns the value of attribute visible_mark.
Class Method Summary collapse
- .[](name) ⇒ Object
- .add(buffer) ⇒ Object
- .auto_detect_encodings ⇒ Object
- .auto_detect_encodings=(encodings) ⇒ Object
- .bury(buffer = @@current) ⇒ Object
- .count ⇒ Object
- .current ⇒ Object
- .current=(buffer) ⇒ Object
- .detect_encoding_proc ⇒ Object
- .detect_encoding_proc=(f) ⇒ Object
- .display_width(s) ⇒ Object
- .dump_unsaved_buffers(dir) ⇒ Object
- .dumped_buffers_exist?(dir) ⇒ Boolean
- .each(&block) ⇒ Object
- .find_file(file_name) ⇒ Object
- .find_or_new(name, **opts) ⇒ Object
- .global_mark_ring ⇒ Object
- .kill_em_all ⇒ Object
- .last ⇒ Object
- .list ⇒ Object
- .load(path) ⇒ Object
- .load_dumped_buffers(dir) ⇒ Object
- .minibuffer ⇒ Object
- .names ⇒ Object
- .new_buffer(name, **opts) ⇒ Object
- .new_buffer_name(name) ⇒ Object
- .open(file_name, name: File.basename(file_name)) ⇒ Object
- .other(buffer = @@current) ⇒ Object
- .region_boundaries(s, e) ⇒ Object
Instance Method Summary collapse
- #[](name) ⇒ Object
- #[]=(name, value) ⇒ Object
- #activate_mark ⇒ Object
- #apply_mode(mode_class) ⇒ Object
- #backward_char(n = 1) ⇒ Object
- #backward_delete_char(n = 1) ⇒ Object
- #backward_line(n = 1) ⇒ Object
- #backward_word(n = 1, regexp: WORD_COMPONENT_REGEXP) ⇒ Object
- #beginning_of_buffer ⇒ Object
- #beginning_of_buffer? ⇒ Boolean
- #beginning_of_line ⇒ Object
- #beginning_of_line? ⇒ Boolean
- #binary? ⇒ Boolean
- #byte_after(location = @point) ⇒ Object
- #byte_before(location = @point) ⇒ Object
- #byteindex(forward, re, pos) ⇒ Object
- #bytesize ⇒ Object (also: #size)
- #capitalize_word(n = 1) ⇒ Object
- #char_after(location = @point) ⇒ Object
- #char_before(location = @point) ⇒ Object
- #clear ⇒ Object
- #composite_edit ⇒ Object
- #convert_word(n = 1, &block) ⇒ Object
- #copy_region(s = @point, e = mark, append = false) ⇒ Object
- #current? ⇒ Boolean
- #current_symbol ⇒ Object
- #deactivate_mark ⇒ Object
- #delete_char(n = 1) ⇒ Object
- #delete_isearch_mark ⇒ Object
- #delete_region(s = @point, e = mark) ⇒ Object
- #delete_visible_mark ⇒ Object
- #disable_input_method ⇒ Object
- #display_width(s) ⇒ Object
- #downcase_word(n = 1) ⇒ Object
- #dump(path) ⇒ Object
- #end_of_buffer ⇒ Object
- #end_of_buffer? ⇒ Boolean
- #end_of_line ⇒ Object
- #end_of_line? ⇒ Boolean
- #exchange_point_and_mark(mark = @mark) ⇒ Object
- #expand_tab(s) ⇒ Object
- #file_modified? ⇒ Boolean
- #filter_event(event) ⇒ Object
- #forward_char(n = 1) ⇒ Object
- #forward_line(n = 1) ⇒ Object
- #forward_word(n = 1, regexp: WORD_COMPONENT_REGEXP) ⇒ Object
- #gap_filled_with_nul? ⇒ Boolean
- #get_line_and_column(pos) ⇒ Object
- #goto_char(pos) ⇒ Object
- #goto_line(n) ⇒ Object
- #gsub(*args, &block) ⇒ Object
- #indent_to(column) ⇒ Object
-
#initialize(s = +"",, name: nil, file_name: nil, file_encoding: , file_mtime: nil, new_file: true, undo_limit: UNDO_LIMIT, read_only: false) ⇒ Buffer
constructor
s might not be copied.
- #input_method_status ⇒ Object
- #insert(x, merge_undo = false) ⇒ Object
- #insert_final_newline ⇒ Object
- #insert_for_yank(s) ⇒ Object
- #inspect ⇒ Object
- #kill ⇒ Object
- #kill_line(append = false) ⇒ Object
- #kill_region(s = @point, e = mark, append = false) ⇒ Object
- #kill_word(append = false) ⇒ Object
- #looking_at?(re) ⇒ Boolean
- #mark ⇒ Object
- #mark_active? ⇒ Boolean
- #mark_to_point(mark) ⇒ Object
- #match_beginning(n) ⇒ Object
- #match_end(n) ⇒ Object
- #match_string(n) ⇒ Object
- #minor_mode_active?(mode_class) ⇒ Boolean
- #mode_names ⇒ Object
- #modified=(modified) ⇒ Object
- #modified? ⇒ Boolean
- #new_file? ⇒ Boolean
- #new_mark(location = @point) ⇒ Object
- #newline ⇒ Object
- #next_line(n = 1) ⇒ Object
- #on(name, &callback) ⇒ Object
- #on_global_mark_ring? ⇒ Boolean
- #on_killed(&callback) ⇒ Object
- #on_modified(&callback) ⇒ Object
- #point_after_mark?(mark) ⇒ Boolean
- #point_at_mark?(mark) ⇒ Boolean
- #point_before_mark?(mark) ⇒ Boolean
- #point_compare_mark(mark) ⇒ Object
- #point_max ⇒ Object
- #point_min ⇒ Object
- #point_to_mark(mark) ⇒ Object
- #pop_mark ⇒ Object
- #pop_to_mark ⇒ Object
- #pos_to_line_and_column(pos) ⇒ Object
- #previous_line(n = 1) ⇒ Object
- #push_global_mark(pos = @point, force: false) ⇒ Object
-
#push_mark(pos = @point) ⇒ Object
Set mark at pos, and push the mark on the mark ring.
- #re_search_backward(s, raise_error: true, count: 1) ⇒ Object
- #re_search_forward(s, raise_error: true, goto_beginning: false, count: 1) ⇒ Object
- #read_only=(value) ⇒ Object
- #read_only? ⇒ Boolean
- #read_only_edit ⇒ Object
- #redo ⇒ Object
- #replace(str = nil, start: point_min, end: point_max, &block) ⇒ Object
- #replace_match(str) ⇒ Object
- #replace_regexp_forward(regexp, to_str) ⇒ Object
- #replace_region(str = nil, &block) ⇒ Object
- #revert(enc = nil) ⇒ Object
- #save(file_name = @file_name) ⇒ Object
-
#save_excursion ⇒ Object
Don’t save Buffer.current.
-
#save_point ⇒ Object
The buffer should not be modified in the given block because current_line/current_column is not updated in save_point.
- #set_isearch_mark(pos = @point) ⇒ Object
- #set_mark(pos = @point) ⇒ Object
- #set_minor_mode(mode_class, arg = nil) ⇒ Object
- #set_read_only(value) ⇒ Object
- #set_visible_mark(pos = @point) ⇒ Object
- #skip_re_backward(re) ⇒ Object
- #skip_re_forward(re) ⇒ Object
- #substring(s, e) ⇒ Object
- #to_s ⇒ Object
- #toggle_input_method(name) ⇒ Object
- #transpose_chars ⇒ Object
- #trim_trailing_whitespace ⇒ Object
- #undo ⇒ Object
- #upcase_word(n = 1) ⇒ Object
- #yank ⇒ Object
- #yank_pop ⇒ Object
Constructor Details
#initialize(s = +"",, name: nil, file_name: nil, file_encoding: , file_mtime: nil, new_file: true, undo_limit: UNDO_LIMIT, read_only: false) ⇒ Buffer
s might not be copied.
221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 |
# File 'lib/textbringer/buffer.rb', line 221 def initialize(s = +"", name: nil, file_name: nil, file_encoding: CONFIG[:default_file_encoding], file_mtime: nil, new_file: true, undo_limit: UNDO_LIMIT, read_only: false) set_contents(s, file_encoding) @name = name @file_name = file_name self.file_encoding = file_encoding @file_mtime = file_mtime @new_file = new_file @undo_limit = undo_limit @point = 0 @gap_start = 0 @gap_end = 0 @marks = [] @mark = nil @mark_ring = Ring.new(CONFIG[:mark_ring_max], on_delete: ->(mark) { mark.delete }) @current_line = 1 @current_column = 1 # One-based character count @goal_column = nil # Zero-based display width count @undo_stack = [] @redo_stack = [] @undoing = false @composite_edit_level = 0 @composite_edit_actions = [] @version = 0 @modified = false @mode = FundamentalMode.new(self) @minor_modes = [] @keymap = nil @attributes = {} @save_point_level = 0 @match_offsets = [] @visible_mark = nil @isearch_mark = nil @mark_active = false @read_only = read_only @callbacks = {} @last_match = nil @input_method = nil end |
Instance Attribute Details
#current_column ⇒ Object (readonly)
Returns the value of attribute current_column.
13 14 15 |
# File 'lib/textbringer/buffer.rb', line 13 def current_column @current_column end |
#current_line ⇒ Object (readonly)
Returns the value of attribute current_line.
13 14 15 |
# File 'lib/textbringer/buffer.rb', line 13 def current_line @current_line end |
#file_encoding ⇒ Object
Returns the value of attribute file_encoding.
12 13 14 |
# File 'lib/textbringer/buffer.rb', line 12 def file_encoding @file_encoding end |
#file_format ⇒ Object
Returns the value of attribute file_format.
12 13 14 |
# File 'lib/textbringer/buffer.rb', line 12 def file_format @file_format end |
#file_name ⇒ Object
Returns the value of attribute file_name.
12 13 14 |
# File 'lib/textbringer/buffer.rb', line 12 def file_name @file_name end |
#input_method ⇒ Object (readonly)
Returns the value of attribute input_method.
15 16 17 |
# File 'lib/textbringer/buffer.rb', line 15 def input_method @input_method end |
#isearch_mark ⇒ Object (readonly)
Returns the value of attribute isearch_mark.
13 14 15 |
# File 'lib/textbringer/buffer.rb', line 13 def isearch_mark @isearch_mark end |
#keymap ⇒ Object
Returns the value of attribute keymap.
11 12 13 |
# File 'lib/textbringer/buffer.rb', line 11 def keymap @keymap end |
#last_match ⇒ Object (readonly)
Returns the value of attribute last_match.
14 15 16 |
# File 'lib/textbringer/buffer.rb', line 14 def last_match @last_match end |
#mark_active ⇒ Object (readonly)
Returns the value of attribute mark_active.
13 14 15 |
# File 'lib/textbringer/buffer.rb', line 13 def mark_active @mark_active end |
#marks ⇒ Object (readonly)
Returns the value of attribute marks.
12 13 14 |
# File 'lib/textbringer/buffer.rb', line 12 def marks @marks end |
#mode ⇒ Object
Returns the value of attribute mode.
11 12 13 |
# File 'lib/textbringer/buffer.rb', line 11 def mode @mode end |
#name ⇒ Object
Returns the value of attribute name.
12 13 14 |
# File 'lib/textbringer/buffer.rb', line 12 def name @name end |
#point ⇒ Object (readonly)
Returns the value of attribute point.
12 13 14 |
# File 'lib/textbringer/buffer.rb', line 12 def point @point end |
#visible_mark ⇒ Object (readonly)
Returns the value of attribute visible_mark.
13 14 15 |
# File 'lib/textbringer/buffer.rb', line 13 def visible_mark @visible_mark end |
Class Method Details
.[](name) ⇒ Object
148 149 150 |
# File 'lib/textbringer/buffer.rb', line 148 def self.[](name) @@table[name] end |
.add(buffer) ⇒ Object
105 106 107 108 |
# File 'lib/textbringer/buffer.rb', line 105 def self.add(buffer) @@table[buffer.name] = buffer @@list.push(buffer) end |
.auto_detect_encodings ⇒ Object
85 86 87 |
# File 'lib/textbringer/buffer.rb', line 85 def self.auto_detect_encodings @@auto_detect_encodings end |
.auto_detect_encodings=(encodings) ⇒ Object
89 90 91 |
# File 'lib/textbringer/buffer.rb', line 89 def self.auto_detect_encodings=(encodings) @@auto_detect_encodings = encodings end |
.bury(buffer = @@current) ⇒ Object
138 139 140 141 142 |
# File 'lib/textbringer/buffer.rb', line 138 def self.bury(buffer = @@current) @@list.delete(buffer) @@list.push(buffer) @@current = @@list.first end |
.count ⇒ Object
144 145 146 |
# File 'lib/textbringer/buffer.rb', line 144 def self.count @@table.size end |
.current ⇒ Object
110 111 112 |
# File 'lib/textbringer/buffer.rb', line 110 def self.current @@current end |
.current=(buffer) ⇒ Object
114 115 116 117 118 119 120 |
# File 'lib/textbringer/buffer.rb', line 114 def self.current=(buffer) if buffer && buffer.name && @@table.key?(buffer.name) @@list.delete(buffer) @@list.unshift(buffer) end @@current = buffer end |
.detect_encoding_proc ⇒ Object
93 94 95 |
# File 'lib/textbringer/buffer.rb', line 93 def self.detect_encoding_proc @@detect_encoding_proc end |
.detect_encoding_proc=(f) ⇒ Object
97 98 99 |
# File 'lib/textbringer/buffer.rb', line 97 def self.detect_encoding_proc=(f) @@detect_encoding_proc = f end |
.display_width(s) ⇒ Object
203 204 205 |
# File 'lib/textbringer/buffer.rb', line 203 def self.display_width(s) Unicode::DisplayWidth.of(s, CONFIG[:east_asian_ambiguous_width]) end |
.dump_unsaved_buffers(dir) ⇒ Object
1393 1394 1395 1396 1397 1398 1399 1400 |
# File 'lib/textbringer/buffer.rb', line 1393 def self.dump_unsaved_buffers(dir) FileUtils.mkdir_p(dir) @@list.each do |buffer| if /\A\*/ !~ buffer.name && buffer.modified? buffer.dump(File.(buffer.object_id.to_s, dir)) end end end |
.dumped_buffers_exist?(dir) ⇒ Boolean
1402 1403 1404 |
# File 'lib/textbringer/buffer.rb', line 1402 def self.dumped_buffers_exist?(dir) !Dir.glob(File.("*.metadata", dir)).empty? end |
.each(&block) ⇒ Object
199 200 201 |
# File 'lib/textbringer/buffer.rb', line 199 def self.each(&block) @@list.each(&block) end |
.find_file(file_name) ⇒ Object
166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 |
# File 'lib/textbringer/buffer.rb', line 166 def self.find_file(file_name) file_name = File.(file_name) buffer = @@table.each_value.find { |b| b.file_name == file_name } if buffer.nil? name = File.basename(file_name) begin buffer = Buffer.open(file_name, name: new_buffer_name(name)) add(buffer) rescue Errno::ENOENT buffer = new_buffer(name, file_name: file_name) end end buffer end |
.find_or_new(name, **opts) ⇒ Object
152 153 154 |
# File 'lib/textbringer/buffer.rb', line 152 def self.find_or_new(name, **opts) @@table[name] ||= new_buffer(name, **opts) end |
.global_mark_ring ⇒ Object
126 127 128 |
# File 'lib/textbringer/buffer.rb', line 126 def self.global_mark_ring @@global_mark_ring ||= Ring.new(CONFIG[:global_mark_ring_max]) end |
.kill_em_all ⇒ Object
160 161 162 163 164 |
# File 'lib/textbringer/buffer.rb', line 160 def self.kill_em_all @@table.clear @@list.clear @@current = nil end |
.last ⇒ Object
134 135 136 |
# File 'lib/textbringer/buffer.rb', line 134 def self.last @@list.last end |
.list ⇒ Object
101 102 103 |
# File 'lib/textbringer/buffer.rb', line 101 def self.list @@list.dup end |
.load(path) ⇒ Object
1382 1383 1384 1385 1386 1387 1388 1389 1390 1391 |
# File 'lib/textbringer/buffer.rb', line 1382 def self.load(path) buffer = Buffer.new(File.binread(path)) = JSON.parse(File.binread(path + ".metadata")) buffer.name = ["name"] buffer.file_name = ["file_name"] if ["file_name"] buffer.file_encoding = Encoding.find(["file_encoding"]) buffer.file_format = ["file_format"].intern buffer.modified = true buffer end |
.load_dumped_buffers(dir) ⇒ Object
1406 1407 1408 1409 1410 1411 1412 1413 1414 1415 |
# File 'lib/textbringer/buffer.rb', line 1406 def self.load_dumped_buffers(dir) Dir.glob(File.("*.metadata", dir)).map do || path = .sub(/\.metadata\z/, "") buffer = Buffer.load(path) add(buffer) File.unlink() File.unlink(path) buffer end end |
.minibuffer ⇒ Object
122 123 124 |
# File 'lib/textbringer/buffer.rb', line 122 def self.minibuffer @@minibuffer ||= Buffer.new(name: "*Minibuffer*") end |
.names ⇒ Object
156 157 158 |
# File 'lib/textbringer/buffer.rb', line 156 def self.names @@table.keys end |
.new_buffer(name, **opts) ⇒ Object
183 184 185 186 187 |
# File 'lib/textbringer/buffer.rb', line 183 def self.new_buffer(name, **opts) buffer = Buffer.new(**opts.merge(name: new_buffer_name(name))) add(buffer) buffer end |
.new_buffer_name(name) ⇒ Object
189 190 191 192 193 194 195 196 197 |
# File 'lib/textbringer/buffer.rb', line 189 def self.new_buffer_name(name) if @@table.key?(name) (2..).lazy.map { |i| "#{name}<#{i}>" }.find { |i| !@@table.key?(i) } else name end end |
.open(file_name, name: File.basename(file_name)) ⇒ Object
393 394 395 396 397 398 399 |
# File 'lib/textbringer/buffer.rb', line 393 def self.open(file_name, name: File.basename(file_name)) buffer = Buffer.new(name: name, file_name: file_name, new_file: false) buffer.revert buffer.read_only = !File.writable?(file_name) buffer end |
.other(buffer = @@current) ⇒ Object
130 131 132 |
# File 'lib/textbringer/buffer.rb', line 130 def self.other(buffer = @@current) @@list.find { |buf| buf != buffer } || Buffer.find_or_new("*scratch*") end |
.region_boundaries(s, e) ⇒ Object
985 986 987 988 989 990 991 |
# File 'lib/textbringer/buffer.rb', line 985 def self.region_boundaries(s, e) if s > e [e, s] else [s, e] end end |
Instance Method Details
#[](name) ⇒ Object
377 378 379 380 381 382 383 |
# File 'lib/textbringer/buffer.rb', line 377 def [](name) if @attributes.key?(name) @attributes[name] else CONFIG[name] end end |
#[]=(name, value) ⇒ Object
385 386 387 |
# File 'lib/textbringer/buffer.rb', line 385 def []=(name, value) @attributes[name] = value end |
#activate_mark ⇒ Object
971 972 973 974 |
# File 'lib/textbringer/buffer.rb', line 971 def activate_mark @mark_active = true set_visible_mark(@mark.location) if @mark end |
#apply_mode(mode_class) ⇒ Object
1318 1319 1320 1321 1322 |
# File 'lib/textbringer/buffer.rb', line 1318 def apply_mode(mode_class) @keymap = nil @mode = mode_class.new(self) Utils.run_hooks(mode_class.hook_name) end |
#backward_char(n = 1) ⇒ Object
691 692 693 |
# File 'lib/textbringer/buffer.rb', line 691 def backward_char(n = 1) forward_char(-n) end |
#backward_delete_char(n = 1) ⇒ Object
680 681 682 |
# File 'lib/textbringer/buffer.rb', line 680 def backward_delete_char(n = 1) delete_char(-n) end |
#backward_line(n = 1) ⇒ Object
741 742 743 |
# File 'lib/textbringer/buffer.rb', line 741 def backward_line(n = 1) forward_line(-n) end |
#backward_word(n = 1, regexp: WORD_COMPONENT_REGEXP) ⇒ Object
707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 |
# File 'lib/textbringer/buffer.rb', line 707 def backward_word(n = 1, regexp: WORD_COMPONENT_REGEXP) return forward_word(-n) if n < 0 n.times do break if beginning_of_buffer? backward_char while !beginning_of_buffer? && regexp !~ char_after backward_char end while !beginning_of_buffer? && regexp =~ char_after backward_char end if regexp !~ char_after forward_char end end end |
#beginning_of_buffer ⇒ Object
766 767 768 769 770 771 772 |
# File 'lib/textbringer/buffer.rb', line 766 def beginning_of_buffer if @save_point_level == 0 @current_line = 1 @current_column = 1 end @point = 0 end |
#beginning_of_buffer? ⇒ Boolean
774 775 776 |
# File 'lib/textbringer/buffer.rb', line 774 def beginning_of_buffer? @point == 0 end |
#beginning_of_line ⇒ Object
786 787 788 789 790 791 |
# File 'lib/textbringer/buffer.rb', line 786 def beginning_of_line while !beginning_of_line? backward_char end @point end |
#beginning_of_line? ⇒ Boolean
793 794 795 |
# File 'lib/textbringer/buffer.rb', line 793 def beginning_of_line? beginning_of_buffer? || byte_before == "\n" end |
#binary? ⇒ Boolean
296 297 298 |
# File 'lib/textbringer/buffer.rb', line 296 def binary? @binary end |
#byte_after(location = @point) ⇒ Object
489 490 491 492 493 494 495 |
# File 'lib/textbringer/buffer.rb', line 489 def byte_after(location = @point) if location < @gap_start @contents.byteslice(location) else @contents.byteslice(location + gap_size) end&.force_encoding(Encoding::ASCII_8BIT) end |
#byte_before(location = @point) ⇒ Object
497 498 499 500 501 502 503 |
# File 'lib/textbringer/buffer.rb', line 497 def byte_before(location = @point) if location <= point_min || location > point_max nil else byte_after(location - 1) end end |
#byteindex(forward, re, pos) ⇒ Object
1198 1199 1200 1201 1202 1203 1204 1205 1206 1207 1208 1209 1210 1211 1212 1213 1214 1215 1216 1217 |
# File 'lib/textbringer/buffer.rb', line 1198 def byteindex(forward, re, pos) @match_offsets = [] @last_match = nil method = forward ? :byteindex : :byterindex adjust_gap(0, 0) s = @contents.byteslice(@gap_end..-1) unless binary? s.force_encoding(Encoding::UTF_8) end i = s.send(method, re, pos) if i @last_match = Regexp.last_match (0 .. @last_match.size - 1).each do |j| @match_offsets.push(@last_match.byteoffset(j)) end i else nil end end |
#bytesize ⇒ Object Also known as: size
527 528 529 |
# File 'lib/textbringer/buffer.rb', line 527 def bytesize @contents.bytesize - gap_size end |
#capitalize_word(n = 1) ⇒ Object
1111 1112 1113 |
# File 'lib/textbringer/buffer.rb', line 1111 def capitalize_word(n = 1) convert_word(n, &:capitalize) end |
#char_after(location = @point) ⇒ Object
505 506 507 508 509 510 511 512 |
# File 'lib/textbringer/buffer.rb', line 505 def char_after(location = @point) if @binary byte_after(location) else s = substring(location, location + UTF8_CHAR_LEN[byte_after(location)]) s.empty? ? nil : s end end |
#char_before(location = @point) ⇒ Object
514 515 516 517 518 519 520 521 522 523 524 525 |
# File 'lib/textbringer/buffer.rb', line 514 def char_before(location = @point) if @binary byte_before(location) else if beginning_of_buffer? nil else pos = get_pos(location, -1) substring(pos, location) end end end |
#clear ⇒ Object
1051 1052 1053 1054 1055 1056 1057 1058 1059 1060 1061 1062 1063 1064 |
# File 'lib/textbringer/buffer.rb', line 1051 def clear check_read_only_flag @contents = String.new @point = @gap_start = @gap_end = 0 @marks.each do |m| m.location = 0 end @current_line = 1 @current_column = 1 @goal_column = nil self.modified = true @undo_stack.clear @redo_stack.clear end |
#composite_edit ⇒ Object
1298 1299 1300 1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 1314 1315 1316 |
# File 'lib/textbringer/buffer.rb', line 1298 def composite_edit location = @point @composite_edit_level += 1 begin yield ensure @composite_edit_level -= 1 if @composite_edit_level == 0 && !@composite_edit_actions.empty? action = CompositeAction.new(self, location) @composite_edit_actions.each do |i| action.add_action(i) end action.version = @composite_edit_actions.first.version push_undo(action) @composite_edit_actions.clear end end fire_callbacks(:modified) end |
#convert_word(n = 1, &block) ⇒ Object
1094 1095 1096 1097 1098 1099 1100 1101 |
# File 'lib/textbringer/buffer.rb', line 1094 def convert_word(n = 1, &block) s = point forward_word(n) e = point s, e = Buffer.region_boundaries(s, e) str = substring(s, e).gsub(/[\p{Letter}\p{Number}]+/, &block) replace(str, start: s, end: e) end |
#copy_region(s = @point, e = mark, append = false) ⇒ Object
993 994 995 996 997 998 999 1000 1001 |
# File 'lib/textbringer/buffer.rb', line 993 def copy_region(s = @point, e = mark, append = false) s, e = Buffer.region_boundaries(s, e) str = substring(s, e) if append && !KILL_RING.empty? KILL_RING.current.concat(str) else KILL_RING.push(str) end end |
#current? ⇒ Boolean
353 354 355 |
# File 'lib/textbringer/buffer.rb', line 353 def current? @@current == self end |
#current_symbol ⇒ Object
1417 1418 1419 1420 1421 |
# File 'lib/textbringer/buffer.rb', line 1417 def current_symbol from = save_point { skip_re_backward(@mode.symbol_pattern); @point } to = save_point { skip_re_forward(@mode.symbol_pattern); @point } from < to ? substring(from, to) : nil end |
#deactivate_mark ⇒ Object
976 977 978 979 |
# File 'lib/textbringer/buffer.rb', line 976 def deactivate_mark @mark_active = false delete_visible_mark end |
#delete_char(n = 1) ⇒ Object
640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 |
# File 'lib/textbringer/buffer.rb', line 640 def delete_char(n = 1) check_read_only_flag adjust_gap s = @point pos = get_pos(@point, n) if n > 0 str = substring(s, pos) # fill the gap with NUL to avoid invalid byte sequence in UTF-8 @contents.bytesplice(@gap_end...user_to_gap(pos), "\0" * (pos - @point)) @gap_end += pos - @point @marks.each do |m| if m.location > pos m.location -= pos - @point elsif m.location > @point m.location = @point end end push_undo(DeleteAction.new(self, s, s, str)) self.modified = true Utils.run_hooks(:after_change_functions, s, s, str) unless @undoing || !current? elsif n < 0 str = substring(pos, s) update_line_and_column(@point, pos) # fill the gap with NUL to avoid invalid byte sequence in UTF-8 @contents.bytesplice(user_to_gap(pos)...@gap_start, "\0" * (@point - pos)) @marks.each do |m| if m.location >= @point m.location -= @point - pos elsif m.location > pos m.location = pos end end @point = @gap_start = pos push_undo(DeleteAction.new(self, s, pos, str)) self.modified = true Utils.run_hooks(:after_change_functions, pos, pos, str) unless @undoing || !current? end @goal_column = nil end |
#delete_isearch_mark ⇒ Object
964 965 966 967 968 969 |
# File 'lib/textbringer/buffer.rb', line 964 def delete_isearch_mark if @isearch_mark @isearch_mark.delete @isearch_mark = nil end end |
#delete_region(s = @point, e = mark) ⇒ Object
1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 |
# File 'lib/textbringer/buffer.rb', line 1008 def delete_region(s = @point, e = mark) check_read_only_flag old_pos = @point s, e = Buffer.region_boundaries(s, e) update_line_and_column(old_pos, s) save_point do str = substring(s, e) @point = s adjust_gap len = e - s # fill the gap with NUL to avoid invalid byte sequence in UTF-8 @contents.bytesplice(@gap_end, len, "\0" * len) @gap_end += len @marks.each do |m| if m.location > e m.location -= len elsif m.location > s m.location = s end end push_undo(DeleteAction.new(self, old_pos, s, str)) self.modified = true Utils.run_hooks(:after_change_functions, s, s, str) unless @undoing || !current? end end |
#delete_visible_mark ⇒ Object
952 953 954 955 956 957 |
# File 'lib/textbringer/buffer.rb', line 952 def delete_visible_mark if @visible_mark @visible_mark.delete @visible_mark = nil end end |
#disable_input_method ⇒ Object
1481 1482 1483 |
# File 'lib/textbringer/buffer.rb', line 1481 def disable_input_method @input_method&.disable end |
#display_width(s) ⇒ Object
216 217 218 |
# File 'lib/textbringer/buffer.rb', line 216 def display_width(s) Buffer.display_width((s)) end |
#downcase_word(n = 1) ⇒ Object
1103 1104 1105 |
# File 'lib/textbringer/buffer.rb', line 1103 def downcase_word(n = 1) convert_word(n, &:downcase) end |
#dump(path) ⇒ Object
1371 1372 1373 1374 1375 1376 1377 1378 1379 1380 |
# File 'lib/textbringer/buffer.rb', line 1371 def dump(path) File.binwrite(path, to_s) = { "name" => name, "file_name" => file_name, "file_encoding" => file_encoding.name, "file_format" => file_format.to_s } File.binwrite(path + ".metadata", .to_json) end |
#end_of_buffer ⇒ Object
778 779 780 |
# File 'lib/textbringer/buffer.rb', line 778 def end_of_buffer goto_char(bytesize) end |
#end_of_buffer? ⇒ Boolean
782 783 784 |
# File 'lib/textbringer/buffer.rb', line 782 def end_of_buffer? @point == bytesize end |
#end_of_line ⇒ Object
797 798 799 800 801 802 |
# File 'lib/textbringer/buffer.rb', line 797 def end_of_line while !end_of_line? forward_char end @point end |
#end_of_line? ⇒ Boolean
804 805 806 |
# File 'lib/textbringer/buffer.rb', line 804 def end_of_line? end_of_buffer? || byte_after == "\n" end |
#exchange_point_and_mark(mark = @mark) ⇒ Object
838 839 840 841 842 843 844 |
# File 'lib/textbringer/buffer.rb', line 838 def exchange_point_and_mark(mark = @mark) if mark.nil? raise EditorError, "The mark is not set" end update_line_and_column(@point, mark.location) @point, mark.location = mark.location, @point end |
#expand_tab(s) ⇒ Object
207 208 209 210 211 212 213 214 |
# File 'lib/textbringer/buffer.rb', line 207 def (s) # TODO: Support multibyte characters tw = self[:tab_width] fmt = "A#{tw}" s.b.gsub(/([^\t]{#{tw}})|([^\t]*)\t/n) { [$+].pack(fmt) }.force_encoding(Encoding::UTF_8) end |
#file_modified? ⇒ Boolean
461 462 463 |
# File 'lib/textbringer/buffer.rb', line 461 def file_modified? !@file_mtime.nil? && File.mtime(@file_name) != @file_mtime end |
#filter_event(event) ⇒ Object
1485 1486 1487 1488 1489 1490 1491 |
# File 'lib/textbringer/buffer.rb', line 1485 def filter_event(event) if @input_method @input_method.filter_event(event) else event end end |
#forward_char(n = 1) ⇒ Object
684 685 686 687 688 689 |
# File 'lib/textbringer/buffer.rb', line 684 def forward_char(n = 1) pos = get_pos(@point, n) update_line_and_column(@point, pos) @point = pos @goal_column = nil end |
#forward_line(n = 1) ⇒ Object
724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 |
# File 'lib/textbringer/buffer.rb', line 724 def forward_line(n = 1) if n > 0 n.times do end_of_line break if end_of_buffer? forward_char end elsif n < 0 (-n).times do beginning_of_line break if beginning_of_buffer? backward_char beginning_of_line end end end |
#forward_word(n = 1, regexp: WORD_COMPONENT_REGEXP) ⇒ Object
695 696 697 698 699 700 701 702 703 704 705 |
# File 'lib/textbringer/buffer.rb', line 695 def forward_word(n = 1, regexp: WORD_COMPONENT_REGEXP) return backward_word(-n) if n < 0 n.times do while !end_of_buffer? && regexp !~ char_after forward_char end while !end_of_buffer? && regexp =~ char_after forward_char end end end |
#gap_filled_with_nul? ⇒ Boolean
1294 1295 1296 |
# File 'lib/textbringer/buffer.rb', line 1294 def gap_filled_with_nul? @contents.byteslice(@gap_start...@gap_end)&.match?(/\A\0*\z/) end |
#get_line_and_column(pos) ⇒ Object
540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 |
# File 'lib/textbringer/buffer.rb', line 540 def get_line_and_column(pos) line = 1 + @contents.byteslice(0...user_to_gap(pos)).count("\n") if pos == point_min column = 1 else begin i = @contents.byterindex("\n", user_to_gap(get_pos(pos, -1))) rescue RangeError i = nil end if i i += 1 else i = 0 end column = 1 + substring(gap_to_user(i), pos).size end [line, column] end |
#goto_char(pos) ⇒ Object
560 561 562 563 564 565 566 567 568 569 570 571 572 573 |
# File 'lib/textbringer/buffer.rb', line 560 def goto_char(pos) return pos if pos == @point if pos < 0 || pos > size raise RangeError, "Out of buffer" end if !@binary && byte_after(pos)&.match?(/[\x80-\xbf]/n) raise ArgumentError, "Position is in the middle of a character" end @goal_column = nil if @save_point_level == 0 @current_line, @current_column = get_line_and_column(pos) end @point = pos end |
#goto_line(n) ⇒ Object
575 576 577 578 579 580 581 582 583 584 585 586 587 588 |
# File 'lib/textbringer/buffer.rb', line 575 def goto_line(n) pos = point_min i = 1 while i < n && pos < @contents.bytesize pos = @contents.byteindex("\n", pos) break if pos.nil? i += 1 pos += 1 end @point = gap_to_user(pos) @current_line = i @current_column = 1 @goal_column = nil end |
#gsub(*args, &block) ⇒ Object
1435 1436 1437 1438 1439 1440 1441 1442 1443 1444 1445 1446 1447 1448 1449 1450 |
# File 'lib/textbringer/buffer.rb', line 1435 def gsub(*args, &block) if block s = to_s.gsub(*args) { |*params| block.binding.eval('->(backref) { $~ = backref }').call($~) block.call(*params) } else s = to_s.gsub(*args) end composite_edit do delete_region(point_min, point_max) insert(s) end self end |
#indent_to(column) ⇒ Object
1362 1363 1364 1365 1366 1367 1368 1369 |
# File 'lib/textbringer/buffer.rb', line 1362 def indent_to(column) s = if self[:indent_tabs_mode] "\t" * (column / self[:tab_width]) + " " * (column % self[:tab_width]) else " " * column end insert(s) end |
#input_method_status ⇒ Object
1493 1494 1495 1496 1497 1498 1499 |
# File 'lib/textbringer/buffer.rb', line 1493 def input_method_status if @input_method&.enabled? @input_method.status else "--" end end |
#insert(x, merge_undo = false) ⇒ Object
590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 |
# File 'lib/textbringer/buffer.rb', line 590 def insert(x, merge_undo = false) s = x.to_s if !binary? && !s.valid_encoding? raise EditorError, "Invalid encoding: #{s.dump}" end check_read_only_flag pos = @point size = s.bytesize adjust_gap(size) @contents.bytesplice(@point, size, s.b) @marks.each do |m| if m.location > @point m.location += size end end @point = @gap_start += size update_line_and_column(pos, @point) unless @undoing if merge_undo && @undo_stack.last.is_a?(InsertAction) @undo_stack.last.merge(s) @redo_stack.clear else push_undo(InsertAction.new(self, pos, s)) end end self.modified = true @goal_column = nil Utils.run_hooks(:after_change_functions, pos, @point, "") unless @undoing || !current? self end |
#insert_final_newline ⇒ Object
1463 1464 1465 1466 1467 1468 1469 1470 |
# File 'lib/textbringer/buffer.rb', line 1463 def insert_final_newline save_excursion do end_of_buffer if char_before != "\n" insert("\n") end end end |
#insert_for_yank(s) ⇒ Object
1115 1116 1117 1118 1119 1120 |
# File 'lib/textbringer/buffer.rb', line 1115 def insert_for_yank(s) if @mark.nil? || !point_at_mark?(@mark) push_mark end insert(s) end |
#inspect ⇒ Object
265 266 267 |
# File 'lib/textbringer/buffer.rb', line 265 def inspect "#<Buffer:#{@name || '0x%x' % object_id}>" end |
#kill ⇒ Object
337 338 339 340 341 342 343 344 345 346 347 |
# File 'lib/textbringer/buffer.rb', line 337 def kill @@table.delete(@name) @@list.delete(self) if @@current == self @@current = nil end @marks.each do |mark| mark.detach end fire_callbacks(:killed) end |
#kill_line(append = false) ⇒ Object
1066 1067 1068 1069 1070 1071 1072 1073 1074 1075 1076 1077 1078 1079 1080 |
# File 'lib/textbringer/buffer.rb', line 1066 def kill_line(append = false) save_point do |saved| if end_of_buffer? raise RangeError, "End of buffer" end if char_after == ?\n forward_char else end_of_line end pos = @point point_to_mark(saved) kill_region(@point, pos, append) end end |
#kill_region(s = @point, e = mark, append = false) ⇒ Object
1003 1004 1005 1006 |
# File 'lib/textbringer/buffer.rb', line 1003 def kill_region(s = @point, e = mark, append = false) copy_region(s, e, append) delete_region(s, e) end |
#kill_word(append = false) ⇒ Object
1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 |
# File 'lib/textbringer/buffer.rb', line 1082 def kill_word(append = false) save_point do |saved| if end_of_buffer? raise RangeError, "End of buffer" end forward_word pos = @point point_to_mark(saved) kill_region(@point, pos, append) end end |
#looking_at?(re) ⇒ Boolean
1189 1190 1191 1192 1193 1194 1195 1196 |
# File 'lib/textbringer/buffer.rb', line 1189 def looking_at?(re) if re.is_a?(Regexp) r = /\G#{re}/ else r = "\\G(?:#{re})" end byteindex(true, r, @point) == @point end |
#mark ⇒ Object
880 881 882 883 884 885 |
# File 'lib/textbringer/buffer.rb', line 880 def mark if @mark.nil? raise EditorError, "The mark is not set" end @mark.location end |
#mark_active? ⇒ Boolean
981 982 983 |
# File 'lib/textbringer/buffer.rb', line 981 def mark_active? @mark_active end |
#mark_to_point(mark) ⇒ Object
818 819 820 |
# File 'lib/textbringer/buffer.rb', line 818 def mark_to_point(mark) mark.location = @point end |
#match_beginning(n) ⇒ Object
1219 1220 1221 |
# File 'lib/textbringer/buffer.rb', line 1219 def match_beginning(n) @match_offsets[n]&.first end |
#match_end(n) ⇒ Object
1223 1224 1225 |
# File 'lib/textbringer/buffer.rb', line 1223 def match_end(n) @match_offsets[n]&.last end |
#match_string(n) ⇒ Object
1227 1228 1229 1230 1231 1232 1233 1234 |
# File 'lib/textbringer/buffer.rb', line 1227 def match_string(n) b, e = @match_offsets[n] if b.nil? nil else substring(b, e) end end |
#minor_mode_active?(mode_class) ⇒ Boolean
1349 1350 1351 |
# File 'lib/textbringer/buffer.rb', line 1349 def minor_mode_active?(mode_class) @minor_modes.any? { |mode| mode.instance_of?(mode_class) } end |
#mode_names ⇒ Object
1353 1354 1355 1356 1357 1358 1359 1360 |
# File 'lib/textbringer/buffer.rb', line 1353 def mode_names names = [] names.push(mode&.name || 'None') @minor_modes.each do |mode| names.push(mode.name) end names end |
#modified=(modified) ⇒ Object
362 363 364 365 366 367 |
# File 'lib/textbringer/buffer.rb', line 362 def modified=(modified) @modified = modified if @composite_edit_level == 0 && modified fire_callbacks(:modified) end end |
#modified? ⇒ Boolean
369 370 371 |
# File 'lib/textbringer/buffer.rb', line 369 def modified? @modified end |
#new_file? ⇒ Boolean
389 390 391 |
# File 'lib/textbringer/buffer.rb', line 389 def new_file? @new_file end |
#new_mark(location = @point) ⇒ Object
808 809 810 811 812 |
# File 'lib/textbringer/buffer.rb', line 808 def new_mark(location = @point) Mark.new(self, location).tap { |m| @marks << m } end |
#newline ⇒ Object
621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 |
# File 'lib/textbringer/buffer.rb', line 621 def newline indentation = save_point { |saved| if char_after&.match?(/[ \t]/) next "" end beginning_of_line s = @point while char_after&.match?(/[ \t]/) forward_char end str = substring(s, @point) if end_of_buffer? || char_after == "\n" delete_region(s, @point) end str } insert("\n" + indentation) end |
#next_line(n = 1) ⇒ Object
745 746 747 748 749 750 751 752 753 |
# File 'lib/textbringer/buffer.rb', line 745 def next_line(n = 1) column = get_goal_column n.times do end_of_line forward_char adjust_column(column) end @goal_column = column end |
#on(name, &callback) ⇒ Object
357 358 359 360 |
# File 'lib/textbringer/buffer.rb', line 357 def on(name, &callback) @callbacks[name] ||= [] @callbacks[name].push(callback) end |
#on_global_mark_ring? ⇒ Boolean
910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 |
# File 'lib/textbringer/buffer.rb', line 910 def on_global_mark_ring? mark_ring = Buffer.global_mark_ring if mark_ring.empty? return false end current = mark_ring.current if current&.buffer == self return true end next_mark = mark_ring[-1] if next_mark&.buffer == self return true end false end |
#on_killed(&callback) ⇒ Object
349 350 351 |
# File 'lib/textbringer/buffer.rb', line 349 def on_killed(&callback) on(:killed, &callback) end |
#on_modified(&callback) ⇒ Object
373 374 375 |
# File 'lib/textbringer/buffer.rb', line 373 def on_modified(&callback) on(:modified, &callback) end |
#point_after_mark?(mark) ⇒ Boolean
830 831 832 |
# File 'lib/textbringer/buffer.rb', line 830 def point_after_mark?(mark) @point > mark.location end |
#point_at_mark?(mark) ⇒ Boolean
822 823 824 |
# File 'lib/textbringer/buffer.rb', line 822 def point_at_mark?(mark) @point == mark.location end |
#point_before_mark?(mark) ⇒ Boolean
826 827 828 |
# File 'lib/textbringer/buffer.rb', line 826 def point_before_mark?(mark) @point < mark.location end |
#point_compare_mark(mark) ⇒ Object
834 835 836 |
# File 'lib/textbringer/buffer.rb', line 834 def point_compare_mark(mark) @point - mark.location end |
#point_max ⇒ Object
536 537 538 |
# File 'lib/textbringer/buffer.rb', line 536 def point_max bytesize end |
#point_min ⇒ Object
532 533 534 |
# File 'lib/textbringer/buffer.rb', line 532 def point_min 0 end |
#point_to_mark(mark) ⇒ Object
814 815 816 |
# File 'lib/textbringer/buffer.rb', line 814 def point_to_mark(mark) goto_char(mark.location) end |
#pop_mark ⇒ Object
937 938 939 940 |
# File 'lib/textbringer/buffer.rb', line 937 def pop_mark return if @mark_ring.empty? @mark = @mark_ring.rotate(1) end |
#pop_to_mark ⇒ Object
942 943 944 945 |
# File 'lib/textbringer/buffer.rb', line 942 def pop_to_mark goto_char(mark) pop_mark end |
#pos_to_line_and_column(pos) ⇒ Object
1501 1502 1503 1504 1505 1506 1507 1508 1509 1510 1511 1512 |
# File 'lib/textbringer/buffer.rb', line 1501 def pos_to_line_and_column(pos) return [1, 1] if pos == 0 text_before = substring(0, pos) line = text_before.count("\n") + 1 last_newline = text_before.rindex("\n") column = if last_newline text_before.size - last_newline else text_before.size + 1 end [line, column] end |
#previous_line(n = 1) ⇒ Object
755 756 757 758 759 760 761 762 763 764 |
# File 'lib/textbringer/buffer.rb', line 755 def previous_line(n = 1) column = get_goal_column n.times do beginning_of_line backward_char beginning_of_line adjust_column(column) end @goal_column = column end |
#push_global_mark(pos = @point, force: false) ⇒ Object
926 927 928 929 930 931 932 933 934 935 |
# File 'lib/textbringer/buffer.rb', line 926 def push_global_mark(pos = @point, force: false) if force || !on_global_mark_ring? mark = new_mark mark.location = pos Buffer.global_mark_ring.push(mark) true else false end end |
#push_mark(pos = @point) ⇒ Object
Set mark at pos, and push the mark on the mark ring. Unlike Emacs, the new mark is pushed on the mark ring instead of the old one.
898 899 900 901 902 903 904 905 906 907 908 |
# File 'lib/textbringer/buffer.rb', line 898 def push_mark(pos = @point) @mark = new_mark @mark.location = pos @mark_ring.push(@mark) if self != Buffer.minibuffer global_mark_ring = Buffer.global_mark_ring if global_mark_ring.empty? || global_mark_ring.current.buffer != self push_global_mark(pos) end end end |
#re_search_backward(s, raise_error: true, count: 1) ⇒ Object
1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 1172 1173 1174 1175 1176 1177 1178 1179 1180 1181 1182 1183 1184 1185 1186 1187 |
# File 'lib/textbringer/buffer.rb', line 1159 def re_search_backward(s, raise_error: true, count: 1) if count < 0 return re_search_forward(s, raise_error: raise_error, count: -count) end re = new_regexp(s) pos = @point count.times do p = pos begin i = byteindex(false, re, p) if i.nil? if raise_error raise SearchError, "Search failed" else return nil end end p = get_pos(p, -1) rescue RangeError if raise_error raise SearchError, "Search failed" else return nil end end while match_end(0) > pos pos = match_beginning(0) end goto_char(pos) end |
#re_search_forward(s, raise_error: true, goto_beginning: false, count: 1) ⇒ Object
1139 1140 1141 1142 1143 1144 1145 1146 1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 |
# File 'lib/textbringer/buffer.rb', line 1139 def re_search_forward(s, raise_error: true, goto_beginning: false, count: 1) if count < 0 return re_search_backward(s, raise_error: raise_error, count: -count) end re = new_regexp(s) pos = @point count.times do i = byteindex(true, re, pos) if i.nil? if raise_error raise SearchError, "Search failed" else return nil end end pos = goto_beginning ? match_beginning(0) : match_end(0) end goto_char(pos) end |
#read_only=(value) ⇒ Object
317 318 319 320 321 322 |
# File 'lib/textbringer/buffer.rb', line 317 def read_only=(value) @read_only = value if @read_only @modified = false end end |
#read_only? ⇒ Boolean
313 314 315 |
# File 'lib/textbringer/buffer.rb', line 313 def read_only? @read_only end |
#read_only_edit ⇒ Object
328 329 330 331 332 333 334 335 |
# File 'lib/textbringer/buffer.rb', line 328 def read_only_edit self.read_only = false begin yield ensure self.read_only = true end end |
#redo ⇒ Object
1135 1136 1137 |
# File 'lib/textbringer/buffer.rb', line 1135 def redo undo_or_redo(:redo, @redo_stack, @undo_stack) end |
#replace(str = nil, start: point_min, end: point_max, &block) ⇒ Object
1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 |
# File 'lib/textbringer/buffer.rb', line 1034 def replace(str = nil, start: point_min, end: point_max, &block) end_pos = binding.local_variable_get(:end) if str.nil? str = block.call(substring(start, end_pos)) end composite_edit do delete_region(start, end_pos) goto_char(start) insert(str) end end |
#replace_match(str) ⇒ Object
1236 1237 1238 1239 1240 1241 1242 1243 1244 1245 1246 1247 1248 1249 1250 1251 1252 1253 1254 |
# File 'lib/textbringer/buffer.rb', line 1236 def replace_match(str) new_str = str.gsub(/\\(?:([0-9]+)|(&)|(\\))/) { |s| case when $1 match_string($1.to_i) when $2 match_string(0) when $3 "\\" end } b = match_beginning(0) e = match_end(0) goto_char(b) composite_edit do delete_region(b, e) insert(new_str) end end |
#replace_regexp_forward(regexp, to_str) ⇒ Object
1256 1257 1258 1259 1260 1261 1262 1263 1264 1265 1266 1267 1268 1269 1270 1271 1272 1273 1274 1275 1276 1277 1278 |
# File 'lib/textbringer/buffer.rb', line 1256 def replace_regexp_forward(regexp, to_str) result = 0 rest = substring(point, point_max) composite_edit do delete_region(point, point_max) new_str = rest.gsub(new_regexp(regexp)) { result += 1 m = Regexp.last_match to_str.gsub(/\\(?:([0-9]+)|(&)|(\\))/) { |s| case when $1 m[$1.to_i] when $2 m.to_s when $3 "\\" end } } insert(new_str) end result end |
#replace_region(str = nil, &block) ⇒ Object
1046 1047 1048 1049 |
# File 'lib/textbringer/buffer.rb', line 1046 def replace_region(str = nil, &block) s, e = Buffer.region_boundaries(@point, mark) replace(str, start: s, end: e, &block) end |
#revert(enc = nil) ⇒ Object
401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 |
# File 'lib/textbringer/buffer.rb', line 401 def revert(enc = nil) if @file_name.nil? raise EditorError, "Buffer has no file name" end clear s, mtime = File.open(@file_name, external_encoding: Encoding::ASCII_8BIT, binmode: true) { |f| f.flock(File::LOCK_SH) [f.read, f.mtime] } enc ||= @@detect_encoding_proc.call(s) || Encoding::ASCII_8BIT s.force_encoding(enc) unless s.valid_encoding? enc = Encoding::ASCII_8BIT s.force_encoding(enc) end set_contents(s, enc) @file_mtime = mtime @modified = false end |
#save(file_name = @file_name) ⇒ Object
423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 |
# File 'lib/textbringer/buffer.rb', line 423 def save(file_name = @file_name) if file_name.nil? raise EditorError, "File name is not set" end file_name = File.(file_name) config = EditorConfig.load_file(file_name) if config["trim_trailing_whitespace"] trim_trailing_whitespace end if config["insert_final_newline"] insert_final_newline end begin File.open(file_name, "w", external_encoding: @file_encoding, binmode: true) do |f| f.flock(File::LOCK_EX) write_to_file(f) f.flush f.fsync end @file_mtime = File.mtime(file_name) rescue Errno::EISDIR if @name file_name = File.(@name, file_name) retry else raise end end if file_name != @file_name self.file_name = file_name end @version += 1 @modified = false @new_file = false @read_only = false end |
#save_excursion ⇒ Object
Don’t save Buffer.current.
863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 |
# File 'lib/textbringer/buffer.rb', line 863 def save_excursion old_point = new_mark old_mark = @mark&.dup old_column = @goal_column begin yield ensure point_to_mark(old_point) old_point.delete if old_mark @mark.location = old_mark.location old_mark.delete end @goal_column = old_column end end |
#save_point ⇒ Object
The buffer should not be modified in the given block because current_line/current_column is not updated in save_point.
848 849 850 851 852 853 854 855 856 857 858 859 860 |
# File 'lib/textbringer/buffer.rb', line 848 def save_point saved = new_mark column = @goal_column @save_point_level += 1 begin yield(saved) ensure point_to_mark(saved) saved.delete @goal_column = column @save_point_level -= 1 end end |
#set_isearch_mark(pos = @point) ⇒ Object
959 960 961 962 |
# File 'lib/textbringer/buffer.rb', line 959 def set_isearch_mark(pos = @point) @isearch_mark ||= new_mark @isearch_mark.location = pos end |
#set_mark(pos = @point) ⇒ Object
887 888 889 890 891 892 893 |
# File 'lib/textbringer/buffer.rb', line 887 def set_mark(pos = @point) if @mark @mark.location = pos else push_mark(pos) end end |
#set_minor_mode(mode_class, arg = nil) ⇒ Object
1324 1325 1326 1327 1328 1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 1346 1347 |
# File 'lib/textbringer/buffer.rb', line 1324 def set_minor_mode(mode_class, arg = nil) mode = @minor_modes.find { |mode| mode.instance_of?(mode_class) } enabled = !!mode enable = case arg when true, false return if enabled == arg arg when nil !enabled else raise ArgumentError, "wrong argument #{arg.inspect} (expected true, false, or nil)" end if enable mode = mode_class.new(self) @minor_modes.push(mode) mode.enable else mode.disable @minor_modes.delete(mode) end end |
#set_read_only(value) ⇒ Object
324 325 326 |
# File 'lib/textbringer/buffer.rb', line 324 def set_read_only(value) @read_only = value end |
#set_visible_mark(pos = @point) ⇒ Object
947 948 949 950 |
# File 'lib/textbringer/buffer.rb', line 947 def set_visible_mark(pos = @point) @visible_mark ||= new_mark @visible_mark.location = pos end |
#skip_re_backward(re) ⇒ Object
1429 1430 1431 1432 1433 |
# File 'lib/textbringer/buffer.rb', line 1429 def skip_re_backward(re) while re =~ char_before backward_char end end |
#skip_re_forward(re) ⇒ Object
1423 1424 1425 1426 1427 |
# File 'lib/textbringer/buffer.rb', line 1423 def skip_re_forward(re) while re =~ char_after forward_char end end |
#substring(s, e) ⇒ Object
474 475 476 477 478 479 480 481 482 483 484 485 486 487 |
# File 'lib/textbringer/buffer.rb', line 474 def substring(s, e) result = if s > @gap_start || e <= @gap_start @contents.byteslice(user_to_gap(s)...user_to_gap(e)) else len = @gap_start - s @contents.byteslice(user_to_gap(s), len) + @contents.byteslice(@gap_end, e - s - len) end if !@binary result.force_encoding(Encoding::UTF_8) end result end |
#to_s ⇒ Object
465 466 467 468 469 470 471 472 |
# File 'lib/textbringer/buffer.rb', line 465 def to_s result = @contents.byteslice(0...@gap_start) + @contents.byteslice(@gap_end..-1) if !@binary result.force_encoding(Encoding::UTF_8) end result end |
#toggle_input_method(name) ⇒ Object
1472 1473 1474 1475 1476 1477 1478 1479 |
# File 'lib/textbringer/buffer.rb', line 1472 def toggle_input_method(name) if name.nil? @input_method ||= InputMethod.find(CONFIG[:default_input_method]) else @input_method = InputMethod.find(name) end @input_method.toggle end |
#transpose_chars ⇒ Object
1280 1281 1282 1283 1284 1285 1286 1287 1288 1289 1290 1291 1292 |
# File 'lib/textbringer/buffer.rb', line 1280 def transpose_chars if end_of_line? backward_char end if beginning_of_buffer? raise RangeError, "Beginning of buffer" end backward_char c = char_after delete_char forward_char insert(c) end |
#trim_trailing_whitespace ⇒ Object
1452 1453 1454 1455 1456 1457 1458 1459 1460 1461 |
# File 'lib/textbringer/buffer.rb', line 1452 def trim_trailing_whitespace save_excursion do beginning_of_buffer composite_edit do while re_search_forward(/[ \t]+$/, raise_error: false) replace_match("") end end end end |
#undo ⇒ Object
1131 1132 1133 |
# File 'lib/textbringer/buffer.rb', line 1131 def undo undo_or_redo(:undo, @undo_stack, @redo_stack) end |
#upcase_word(n = 1) ⇒ Object
1107 1108 1109 |
# File 'lib/textbringer/buffer.rb', line 1107 def upcase_word(n = 1) convert_word(n, &:upcase) end |
#yank ⇒ Object
1122 1123 1124 |
# File 'lib/textbringer/buffer.rb', line 1122 def yank insert_for_yank(KILL_RING.current) end |
#yank_pop ⇒ Object
1126 1127 1128 1129 |
# File 'lib/textbringer/buffer.rb', line 1126 def yank_pop delete_region insert_for_yank(KILL_RING.rotate(1)) end |