Class: Textbringer::Window

Inherits:
Object
  • Object
show all
Defined in:
lib/textbringer/window.rb,
lib/textbringer/window/fallback_characters.rb

Direct Known Subclasses

EchoArea, FloatingWindow

Defined Under Namespace

Classes: Cursor

Constant Summary collapse

KEY_NAMES =
{}
HAVE_GET_KEY_MODIFIERS =
defined?(Curses.get_key_modifiers)
ALT_NUMBER_BASE =
Curses::ALT_0 - ?0.ord
ALT_ALPHA_BASE =
Curses::ALT_A - ?a.ord
FALLBACK_CHARACTERS =
{
  # Diacritical marks
  ## https://www.unicode.org/charts/PDF/U0300.pdf
  ## characters combinable with alphabet:
  ## ("\u{0300}".."\u{036f}").filter { |c|
  ##   ("a".."z").any? { |c2|
  ##     "#{c2}#{c}".unicode_normalize(:nfc).size == 1
  ##   }
  ## }
  "̀" => "`",
  "́" => "´",
  "̂" => "ˆ",
  "̃" => "˜",
  "̄" => "¯",
  "̆" => "˘",
  "̇" => "˙",
  "̈" => "¨",
  "̉" => "ˀ",
  "̊" => "˚",
  "̋" => "˝",
  "̌" => "ˇ",
  "̏" => '"',
  "̑" => nil, # combining character only
  "̛" => nil, # combining character only
  "̣" => nil, # combining character only
  "̤" => nil, # combining character only
  "̥" => "˳",
  "̦" => ",",
  "̧" => "¸",
  "̨" => "˛",
  "̭" => nil, # combining character only
  "̮" => nil, # combining character only
  "̰" => nil, # combining character only
  "̱" => "ˍ",
  "̀" => nil, # combining character only
  "́" => nil, # combining character only
  "̈́" => nil, # combining character only

  # Hiragana
  "" => "",
  "" => "",

  # Hangul jamo
  ## initial
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  ## medial
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  ## final
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => "",
  "" => ""
}
@@started =
false
@@list =
[]
@@current =
nil
@@minibuffer_selected =
nil
@@echo_area =
nil
@@has_colors =
false

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(lines, columns, y, x) ⇒ Window

Returns a new instance of Window.



259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
# File 'lib/textbringer/window.rb', line 259

def initialize(lines, columns, y, x)
  @lines = lines
  @columns = columns
  @y = y
  @x = x
  initialize_window(lines, columns, y, x)
  @window.keypad = true
  @window.scrollok(false)
  @window.idlok(true)
  @buffer = nil
  @top_of_window = nil
  @bottom_of_window = nil
  @point_mark = nil
  @deleted = false
  @raw_key_buffer = []
  @key_buffer = []
  @cursor = Cursor.new(0, 0)
  @in_region = false
  @in_isearch = false
  @current_hl_face = nil
end

Instance Attribute Details

#bottom_of_windowObject (readonly)

Returns the value of attribute bottom_of_window.



257
258
259
# File 'lib/textbringer/window.rb', line 257

def bottom_of_window
  @bottom_of_window
end

#bufferObject

Returns the value of attribute buffer.



256
257
258
# File 'lib/textbringer/window.rb', line 256

def buffer
  @buffer
end

#columnsObject (readonly)

Returns the value of attribute columns.



256
257
258
# File 'lib/textbringer/window.rb', line 256

def columns
  @columns
end

#cursorObject (readonly)

Returns the value of attribute cursor.



257
258
259
# File 'lib/textbringer/window.rb', line 257

def cursor
  @cursor
end

#linesObject (readonly)

Returns the value of attribute lines.



256
257
258
# File 'lib/textbringer/window.rb', line 256

def lines
  @lines
end

#mode_lineObject (readonly)

Returns the value of attribute mode_line.



256
257
258
# File 'lib/textbringer/window.rb', line 256

def mode_line
  @mode_line
end

#top_of_windowObject (readonly)

Returns the value of attribute top_of_window.



257
258
259
# File 'lib/textbringer/window.rb', line 257

def top_of_window
  @top_of_window
end

#windowObject (readonly)

Returns the value of attribute window.



256
257
258
# File 'lib/textbringer/window.rb', line 256

def window
  @window
end

#xObject (readonly)

Returns the value of attribute x.



256
257
258
# File 'lib/textbringer/window.rb', line 256

def x
  @x
end

#yObject (readonly)

Returns the value of attribute y.



256
257
258
# File 'lib/textbringer/window.rb', line 256

def y
  @y
end

Class Method Details

.beepObject



252
253
254
# File 'lib/textbringer/window.rb', line 252

def self.beep
  Curses.beep
end

.colorsObject



131
132
133
# File 'lib/textbringer/window.rb', line 131

def self.colors
  Curses.colors
end

.columnsObject



225
226
227
# File 'lib/textbringer/window.rb', line 225

def self.columns
  Curses.cols
end

.currentObject Also known as: selected



45
46
47
# File 'lib/textbringer/window.rb', line 45

def self.current
  @@current
end

.current=(window) ⇒ Object



49
50
51
52
53
54
55
56
57
# File 'lib/textbringer/window.rb', line 49

def self.current=(window)
  if window.deleted?
    window = @@list.first
  end
  @@current.save_point if @@current && !@@current.deleted?
  @@current = window
  @@current.restore_point
  Buffer.current = window.buffer
end

.delete_other_windowsObject



94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
# File 'lib/textbringer/window.rb', line 94

def self.delete_other_windows
  if @@current.echo_area?
    raise EditorError, "Can't expand the echo area to full screen"
  end
  @@list.delete_if do |window|
    if window.current? || window.echo_area?
      false
    else
      window.delete
      true
    end
  end
  @@current.move(0, 0)
  @@current.resize(Window.lines - 1, @@current.columns)
end

.delete_window(target = @@current) ⇒ Object



71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
# File 'lib/textbringer/window.rb', line 71

def self.delete_window(target = @@current)
  if target.echo_area?
    raise EditorError, "Can't delete the echo area"
  end
  if @@list.size == 2
    raise EditorError, "Can't delete the sole window"
  end
  i = @@list.index(target)
  return if i.nil?
  if i == 0
    window = @@list[1]
    window.move(0, 0)
  else
    window = @@list[i - 1]
  end
  window.resize(target.lines + window.lines, window.columns)
  target.delete
  @@list.delete_at(i)
  if target == @@current
    self.current = window
  end
end

.echo_areaObject



119
120
121
# File 'lib/textbringer/window.rb', line 119

def self.echo_area
  @@echo_area
end

.has_colors=(value) ⇒ Object



123
124
125
# File 'lib/textbringer/window.rb', line 123

def self.has_colors=(value)
  @@has_colors = value
end

.has_colors?Boolean

Returns:

  • (Boolean)


127
128
129
# File 'lib/textbringer/window.rb', line 127

def self.has_colors?
  @@has_colors
end

.linesObject



221
222
223
# File 'lib/textbringer/window.rb', line 221

def self.lines
  Curses.lines
end

.list(include_echo_area: false) ⇒ Object



37
38
39
40
41
42
43
# File 'lib/textbringer/window.rb', line 37

def self.list(include_echo_area: false)
  if include_echo_area
    @@list.dup
  else
    @@list.reject(&:echo_area?)
  end
end

.load_facesObject



140
141
142
143
144
# File 'lib/textbringer/window.rb', line 140

def self.load_faces
  require_relative "faces/basic"
  require_relative "faces/programming"
  require_relative "faces/completion"
end

.minibuffer_selectedObject



63
64
65
# File 'lib/textbringer/window.rb', line 63

def self.minibuffer_selected
  @@minibuffer_selected
end

.minibuffer_selected=(window) ⇒ Object



67
68
69
# File 'lib/textbringer/window.rb', line 67

def self.minibuffer_selected=(window)
  @@minibuffer_selected = window
end

.other_windowObject



110
111
112
113
114
115
116
117
# File 'lib/textbringer/window.rb', line 110

def self.other_window
  i = @@list.index(@@current)
  begin
    i += 1
    window = @@list[i % @@list.size]
  end while !window.active?
  self.current = window
end

.redisplayObject



190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
# File 'lib/textbringer/window.rb', line 190

def self.redisplay
  return if Controller.current.executing_keyboard_macro?
  return if Window.current.has_input?
  @@list.each do |window|
    window.redisplay unless window.current?
  end
  current.redisplay
  if defined?(FloatingWindow)
    # Render floating windows on top
    FloatingWindow.redisplay_all_floating
    # Ensure cursor position is from the current window, not floating windows
    # Refresh current window's cursor position after floating windows
    if !FloatingWindow.floating_windows.empty?
      current&.window&.noutrefresh
    end
  end
  update
end

.redrawObject



209
210
211
212
213
214
215
# File 'lib/textbringer/window.rb', line 209

def self.redraw
  @@list.each do |window|
    window.redraw unless window.current?
  end
  current.redraw
  update
end

.resizeObject



229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
# File 'lib/textbringer/window.rb', line 229

def self.resize
  @@list.delete_if do |window|
    if !window.echo_area? &&
        window.y > Window.lines - CONFIG[:window_min_height]
      window.delete
      true
    else
      false
    end
  end
  @@list.each_with_index do |window, i|
    unless window.echo_area?
      if i < @@list.size - 2
        window.resize(window.lines, Window.columns)
      else
        window.resize(Window.lines - 1 - window.y, Window.columns)
      end
    end
  end
  @@echo_area.move(Window.lines - 1, 0)
  @@echo_area.resize(1, Window.columns)
end

.set_default_colors(fg, bg) ⇒ Object



135
136
137
138
# File 'lib/textbringer/window.rb', line 135

def self.set_default_colors(fg, bg)
  Curses.assume_default_colors(Color[fg], Color[bg])
  Window.redraw
end

.startObject



146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
# File 'lib/textbringer/window.rb', line 146

def self.start
  if @@started
    raise EditorError, "Already started"
  end
  Curses.init_screen
  Curses.noecho
  Curses.raw
  Curses.nonl
  self.has_colors = Curses.has_colors?
  if has_colors?
    Curses.start_color
    Curses.use_default_colors
    load_faces
  else
    Face.define :mode_line, reverse: true
    Face.define :region, reverse: true
    Face.define :isearch, underline: true
  end
  begin
    window =
      Textbringer::Window.new(Window.lines - 1, Window.columns, 0, 0)
    window.buffer = Buffer.new_buffer("*scratch*")
    @@list.push(window)
    Window.current = window
    @@echo_area = Textbringer::EchoArea.new(1, Window.columns,
                                            Window.lines - 1, 0)
    Buffer.minibuffer.keymap = MINIBUFFER_LOCAL_MAP
    @@echo_area.buffer = Buffer.minibuffer
    @@list.push(@@echo_area)
    @@started = true
    yield
  ensure
    @@list.each do |win|
      win.close
    end
    @@list.clear
    Curses.echo
    Curses.noraw
    Curses.nl
    Curses.close_screen
    @@started = false
  end
end

.updateObject



217
218
219
# File 'lib/textbringer/window.rb', line 217

def self.update
  Curses.doupdate
end

Instance Method Details

#active?Boolean

Returns:

  • (Boolean)


285
286
287
# File 'lib/textbringer/window.rb', line 285

def active?
  true
end

#closeObject



304
305
306
# File 'lib/textbringer/window.rb', line 304

def close
  @window.close
end

#current?Boolean

Returns:

  • (Boolean)


334
335
336
# File 'lib/textbringer/window.rb', line 334

def current?
  self == @@current
end

#current_or_minibuffer_selected?Boolean

Returns:

  • (Boolean)


338
339
340
# File 'lib/textbringer/window.rb', line 338

def current_or_minibuffer_selected?
  self == @@current || self == @@minibuffer_selected
end

#deleteObject



293
294
295
296
297
298
299
300
301
302
# File 'lib/textbringer/window.rb', line 293

def delete
  unless @deleted
    if current?
      Window.current = @@list.first
    end
    delete_marks
    @window.close
    @deleted = true
  end
end

#deleted?Boolean

Returns:

  • (Boolean)


289
290
291
# File 'lib/textbringer/window.rb', line 289

def deleted?
  @deleted
end

#echo_area?Boolean

Returns:

  • (Boolean)


281
282
283
# File 'lib/textbringer/window.rb', line 281

def echo_area?
  false
end

#enlarge(n) ⇒ Object



617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
# File 'lib/textbringer/window.rb', line 617

def enlarge(n)
  if n > 0
    max_height = Window.lines -
      CONFIG[:window_min_height] * (@@list.size - 2) - 1
    new_lines = [lines + n, max_height].min
    needed_lines = new_lines - lines
    resize(new_lines, columns)
    i = @@list.index(self)
    indices = (i + 1).upto(@@list.size - 2).to_a +
      (i - 1).downto(0).to_a
    indices.each do |j|
      break if needed_lines == 0
      window = @@list[j]
      extended_lines = [
        window.lines - CONFIG[:window_min_height],
        needed_lines
      ].min
      window.resize(window.lines - extended_lines, window.columns)
      needed_lines -= extended_lines
    end
    y = 0
    @@list.each do |win|
      win.move(y, win.x)
      y += win.lines
    end
  elsif n < 0 && @@list.size > 2
    new_lines = [lines + n, CONFIG[:window_min_height]].max
    diff = lines - new_lines
    resize(new_lines, columns)
    i = @@list.index(self)
    if i < @@list.size - 2
      window = @@list[i + 1]
      window.move(window.y - diff, window.x)
    else
      window = @@list[i - 1]
      move(self.y + diff, self.x)
    end
    window.resize(window.lines + diff, window.columns)
  end
end

#has_input?Boolean

Returns:

  • (Boolean)


385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
# File 'lib/textbringer/window.rb', line 385

def has_input?
  if !@raw_key_buffer.empty? || !@key_buffer.empty?
    return true
  end
  @window.nodelay = true
  begin
    c = @window.get_char
    if c
      @raw_key_buffer.push(c)
    end
    !c.nil?
  ensure
    @window.nodelay = false
  end
end

#highlightObject



401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
# File 'lib/textbringer/window.rb', line 401

def highlight
  @highlight_on = {}
  @highlight_off = {}
  return if !@@has_colors || !CONFIG[:syntax_highlight] || @buffer.binary?
  syntax_table = @buffer.mode.syntax_table || DEFAULT_SYNTAX_TABLE
  if @buffer.bytesize < CONFIG[:highlight_buffer_size_limit]
    base_pos = @buffer.point_min
    s = @buffer.to_s
  else
    base_pos = @buffer.point
    len = columns * (lines - 1) / 2 * 3
    s = @buffer.substring(@buffer.point, @buffer.point + len).scrub("")
  end
  return if !s.valid_encoding?
  re_str = syntax_table.map { |name, re|
    "(?<#{name}>#{re})"
  }.join("|")
  re = Regexp.new(re_str)
  names = syntax_table.keys
  s.scan(re) do
    b = base_pos + $`.bytesize
    e = b + $&.bytesize
    if b < @buffer.point && @buffer.point < e
      b = @buffer.point
    end
    name = names.find { |n| $~[n] }
    face = Face[name]
    if face
      @highlight_on[b] = face
      @highlight_off[e] = true
    end
  end
end

#move(y, x) ⇒ Object



542
543
544
545
546
547
# File 'lib/textbringer/window.rb', line 542

def move(y, x)
  @y = y
  @x = x
  @window.move(y, x)
  @mode_line.move(y + @window.maxy, x)
end

#read_eventObject



342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
# File 'lib/textbringer/window.rb', line 342

def read_event
  key = get_char
  if key.is_a?(Integer)
    if HAVE_GET_KEY_MODIFIERS
      if Curses::ALT_0 <= key && key <= Curses::ALT_9
        @key_buffer.push((key - ALT_NUMBER_BASE).chr)
        return "\e"
      elsif Curses::ALT_A <= key && key <= Curses::ALT_Z
        @key_buffer.push((key - ALT_ALPHA_BASE).chr)
        return "\e"
      end
    end
    KEY_NAMES[key] || key
  else
    key&.encode(Encoding::UTF_8)
  end
end

#read_event_nonblockObject



360
361
362
363
364
365
366
367
# File 'lib/textbringer/window.rb', line 360

def read_event_nonblock
  @window.nodelay = true
  begin
    read_event
  ensure
    @window.nodelay = false
  end
end

#recenterObject



557
558
559
560
561
562
563
564
565
566
567
568
# File 'lib/textbringer/window.rb', line 557

def recenter
  @buffer.save_point do |saved|
    max = (lines - 1) / 2
    count = beginning_of_line_and_count(max)
    while count < max
      break if @buffer.point == 0
      @buffer.backward_char
      count += beginning_of_line_and_count(max - count - 1) + 1
    end
    @buffer.mark_to_point(@top_of_window)
  end
end

#recenter_if_neededObject



570
571
572
573
574
575
# File 'lib/textbringer/window.rb', line 570

def recenter_if_needed
  if @buffer.point_before_mark?(@top_of_window) ||
     @buffer.point_after_mark?(@bottom_of_window)
    recenter
  end
end

#redisplayObject



435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
# File 'lib/textbringer/window.rb', line 435

def redisplay
  return if @buffer.nil?
  redisplay_mode_line
  @buffer.save_point do |saved|
    if current?
      point = saved
    else
      point = @point_mark
      @buffer.point_to_mark(@point_mark)
    end
    framer
    @cursor.y = @cursor.x = 0
    @buffer.point_to_mark(@top_of_window)
    highlight
    @window.erase
    @window.setpos(0, 0)
    @in_region = false
    @in_isearch = false
    @current_hl_face = nil
    if current_or_minibuffer_selected? && @buffer.visible_mark &&
       @buffer.point_after_mark?(@buffer.visible_mark)
      @in_region = true
    end
    if current_or_minibuffer_selected? && @buffer.isearch_mark &&
       @buffer.point_after_mark?(@buffer.isearch_mark)
      @in_region = false
      @in_isearch = true
    end
    apply_window_attrs(@window)
    while !@buffer.end_of_buffer?
      cury = @window.cury
      curx = @window.curx
      update_cursor_and_attr(point, cury, curx)
      if @highlight_off[@buffer.point]
        @current_hl_face = nil
        apply_window_attrs(@window)
      end
      if face = @highlight_on[@buffer.point]
        @current_hl_face = face
        apply_window_attrs(@window)
      end
      c = @buffer.char_after
      if c == "\n"
        # Fill to end of line with region background if in region
        if @in_region
          remaining = columns - curx
          @window.addstr(" " * remaining) if remaining > 0
        else
          @window.clrtoeol
        end
        break if cury == lines - 2   # lines include mode line
        @window.setpos(cury + 1, 0)
        @buffer.forward_char
        next
      elsif c == "\t"
        n = calc_tab_width(curx)
        c = " " * n
      else
        c = compose_character(point, cury, curx, c)
      end
      s = escape(c)
      if curx < columns - 4
        newx = nil
      else
        newx = curx + Buffer.display_width(s)
        if newx > columns
          if cury == lines - 2
            break
          else
            # Fill to end of line with region background if in region
            if @in_region
              remaining = columns - curx
              @window.addstr(" " * remaining) if remaining > 0
            else
              @window.clrtoeol
            end
            @window.setpos(cury + 1, 0)
          end
        end
      end
      @window.addstr(s)
      break if newx == columns && cury == lines - 2
      @buffer.forward_char
    end
    apply_window_attrs(@window)
    @buffer.mark_to_point(@bottom_of_window)
    if @buffer.point_at_mark?(point)
      @cursor.y = @window.cury
      @cursor.x = @window.curx
    end
    if @cursor.x == columns - 1
      c = @buffer.char_after(point.location)
      if c && Buffer.display_width(c) > 1
        @cursor.y += 1
        @cursor.x = 0
      end
    end
    @window.setpos(@cursor.y, @cursor.x)
    @window.noutrefresh
  end
end

#redrawObject



537
538
539
540
# File 'lib/textbringer/window.rb', line 537

def redraw
  @window.redraw
  @mode_line.redraw
end

#resize(lines, columns) ⇒ Object



549
550
551
552
553
554
555
# File 'lib/textbringer/window.rb', line 549

def resize(lines, columns)
  @lines = lines
  @columns = columns
  @window.resize(lines - 1, columns)
  @mode_line.move(@y + lines - 1, @x)
  @mode_line.resize(1, columns)
end

#restore_pointObject



330
331
332
# File 'lib/textbringer/window.rb', line 330

def restore_point
  @buffer.point_to_mark(@point_mark)
end

#save_pointObject



322
323
324
325
326
327
328
# File 'lib/textbringer/window.rb', line 322

def save_point
  @buffer[:top_of_window] ||= @buffer.new_mark
  @buffer[:top_of_window].location = @top_of_window.location
  @buffer[:bottom_of_window] ||= @buffer.new_mark
  @buffer[:bottom_of_window].location = @bottom_of_window.location
  @buffer.mark_to_point(@point_mark)
end

#scroll_downObject



587
588
589
590
591
592
593
594
595
# File 'lib/textbringer/window.rb', line 587

def scroll_down
  if @top_of_window.location == @buffer.point_min
    raise RangeError, "Beginning of buffer"
  end
  @buffer.point_to_mark(@top_of_window)
  @buffer.next_line
  @buffer.beginning_of_line
  @top_of_window.location = 0
end

#scroll_upObject



577
578
579
580
581
582
583
584
585
# File 'lib/textbringer/window.rb', line 577

def scroll_up
  if @bottom_of_window.location == @buffer.point_max
    raise RangeError, "End of buffer"
  end
  @buffer.point_to_mark(@bottom_of_window)
  @buffer.previous_line
  @buffer.beginning_of_line
  @buffer.mark_to_point(@top_of_window)
end

#shrink(n) ⇒ Object



658
659
660
# File 'lib/textbringer/window.rb', line 658

def shrink(n)
  enlarge(-n)
end

#shrink_if_larger_than_bufferObject



662
663
664
665
666
667
668
669
670
671
672
673
674
675
# File 'lib/textbringer/window.rb', line 662

def shrink_if_larger_than_buffer
  @buffer.save_point do
    @buffer.end_of_buffer
    @buffer.skip_re_backward(/\s/)
    count = beginning_of_line_and_count(Window.lines) + 1
    while !@buffer.beginning_of_buffer?
      @buffer.backward_char
      count += beginning_of_line_and_count(Window.lines) + 1
    end
    if lines - 1 > count
      shrink(lines - 1 - count)
    end
  end
end

#split(other_lines = nil) ⇒ Object



597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
# File 'lib/textbringer/window.rb', line 597

def split(other_lines = nil)
  old_lines = lines
  if other_lines
    if other_lines < CONFIG[:window_min_height]
      raise EditorError, "Window too small"
    end
    new_lines = lines - other_lines
  else
    new_lines = (old_lines / 2.0).ceil
  end
  if new_lines < CONFIG[:window_min_height]
    raise EditorError, "Window too small"
  end
  resize(new_lines, columns)
  new_window = Window.new(old_lines - new_lines, columns, y + new_lines, x)
  new_window.buffer = buffer
  i = @@list.index(self)
  @@list.insert(i + 1, new_window)
end

#wait_input(msecs) ⇒ Object



369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
# File 'lib/textbringer/window.rb', line 369

def wait_input(msecs)
  if !@raw_key_buffer.empty? || !@key_buffer.empty?
    return @raw_key_buffer.first || @key_buffer.first
  end
  @window.timeout = msecs
  begin
    c = @window.get_char
    if c
      @raw_key_buffer.push(c)
    end
    c
  ensure
    @window.timeout = -1
  end
end