Class: Textbringer::Window
- Inherits:
-
Object
- Object
- Textbringer::Window
- Defined in:
- lib/textbringer/window.rb,
lib/textbringer/window/fallback_characters.rb
Direct Known Subclasses
Defined Under Namespace
Classes: Cursor
Constant Summary collapse
- KEY_NAMES =
{}
- HAVE_GET_KEY_MODIFIERS =
defined?(Curses.get_key_modifiers)
- ALT_NUMBER_BASE =
Curses::ALT_0 - ?0.ord
- ALT_ALPHA_BASE =
Curses::ALT_A - ?a.ord
- FALLBACK_CHARACTERS =
{ # Diacritical marks ## https://www.unicode.org/charts/PDF/U0300.pdf ## characters combinable with alphabet: ## ("\u{0300}".."\u{036f}").filter { |c| ## ("a".."z").any? { |c2| ## "#{c2}#{c}".unicode_normalize(:nfc).size == 1 ## } ## } "̀" => "`", "́" => "´", "̂" => "ˆ", "̃" => "˜", "̄" => "¯", "̆" => "˘", "̇" => "˙", "̈" => "¨", "̉" => "ˀ", "̊" => "˚", "̋" => "˝", "̌" => "ˇ", "̏" => '"', "̑" => nil, # combining character only "̛" => nil, # combining character only "̣" => nil, # combining character only "̤" => nil, # combining character only "̥" => "˳", "̦" => ",", "̧" => "¸", "̨" => "˛", "̭" => nil, # combining character only "̮" => nil, # combining character only "̰" => nil, # combining character only "̱" => "ˍ", "̀" => nil, # combining character only "́" => nil, # combining character only "̈́" => nil, # combining character only # Hiragana "゙" => "゛", "゚" => "゜", # Hangul jamo ## initial "ᄀ" => "ㄱ", "ᄁ" => "ㄲ", "ᄂ" => "ㄴ", "ᄃ" => "ㄷ", "ᄄ" => "ㄸ", "ᄅ" => "ㄹ", "ᄆ" => "ㅁ", "ᄇ" => "ㅂ", "ᄈ" => "ㅃ", "ᄉ" => "ㅅ", "ᄊ" => "ㅆ", "ᄋ" => "ㅇ", "ᄌ" => "ㅈ", "ᄍ" => "ㅉ", "ᄎ" => "ㅊ", "ᄏ" => "ㅋ", "ᄐ" => "ㅌ", "ᄑ" => "ㅍ", "ᄒ" => "ㅎ", ## medial "ᅡ" => "ㅏ", "ᅢ" => "ㅐ", "ᅣ" => "ㅑ", "ᅤ" => "ㅒ", "ᅥ" => "ㅓ", "ᅦ" => "ㅔ", "ᅧ" => "ㅕ", "ᅨ" => "ㅖ", "ᅩ" => "ㅗ", "ᅪ" => "ㅘ", "ᅫ" => "ㅙ", "ᅬ" => "ㅚ", "ᅭ" => "ㅛ", "ᅮ" => "ㅜ", "ᅯ" => "ㅝ", "ᅰ" => "ㅞ", "ᅱ" => "ㅟ", "ᅲ" => "ㅠ", "ᅳ" => "ㅡ", "ᅴ" => "ㅢ", "ᅵ" => "ㅣ", ## final "ᆨ" => "ㄱ", "ᆩ" => "ㄲ", "ᆪ" => "ㄳ", "ᆫ" => "ㄴ", "ᆬ" => "ㄵ", "ᆭ" => "ㄶ", "ᆮ" => "ㄷ", "ᆯ" => "ㄹ", "ᆰ" => "ㄺ", "ᆱ" => "ㄻ", "ᆲ" => "ㄼ", "ᆳ" => "ㄽ", "ᆴ" => "ㄾ", "ᆵ" => "ㄿ", "ᆶ" => "ㅀ", "ᆷ" => "ㅁ", "ᆸ" => "ㅂ", "ᆹ" => "ㅄ", "ᆺ" => "ㅅ", "ᆻ" => "ㅆ", "ᆼ" => "ㅇ", "ᆽ" => "ㅈ", "ᆾ" => "ㅊ", "ᆿ" => "ㅋ", "ᇀ" => "ㅌ", "ᇁ" => "ㅍ", "ᇂ" => "ㅎ" }
- @@started =
false- @@list =
[]
- @@current =
nil- @@minibuffer_selected =
nil- @@echo_area =
nil- @@has_colors =
false
Instance Attribute Summary collapse
-
#bottom_of_window ⇒ Object
readonly
Returns the value of attribute bottom_of_window.
-
#buffer ⇒ Object
Returns the value of attribute buffer.
-
#columns ⇒ Object
readonly
Returns the value of attribute columns.
-
#cursor ⇒ Object
readonly
Returns the value of attribute cursor.
-
#lines ⇒ Object
readonly
Returns the value of attribute lines.
-
#mode_line ⇒ Object
readonly
Returns the value of attribute mode_line.
-
#top_of_window ⇒ Object
readonly
Returns the value of attribute top_of_window.
-
#window ⇒ Object
readonly
Returns the value of attribute window.
-
#x ⇒ Object
readonly
Returns the value of attribute x.
-
#y ⇒ Object
readonly
Returns the value of attribute y.
Class Method Summary collapse
- .beep ⇒ Object
- .colors ⇒ Object
- .columns ⇒ Object
- .current ⇒ Object (also: selected)
- .current=(window) ⇒ Object
- .delete_other_windows ⇒ Object
- .delete_window(target = @@current) ⇒ Object
- .echo_area ⇒ Object
- .has_colors=(value) ⇒ Object
- .has_colors? ⇒ Boolean
- .lines ⇒ Object
- .list(include_echo_area: false) ⇒ Object
- .load_faces ⇒ Object
- .minibuffer_selected ⇒ Object
- .minibuffer_selected=(window) ⇒ Object
- .other_window ⇒ Object
- .redisplay ⇒ Object
- .redraw ⇒ Object
- .resize ⇒ Object
- .set_default_colors(fg, bg) ⇒ Object
- .start ⇒ Object
- .update ⇒ Object
Instance Method Summary collapse
- #active? ⇒ Boolean
- #close ⇒ Object
- #current? ⇒ Boolean
- #current_or_minibuffer_selected? ⇒ Boolean
- #delete ⇒ Object
- #deleted? ⇒ Boolean
- #echo_area? ⇒ Boolean
- #enlarge(n) ⇒ Object
- #has_input? ⇒ Boolean
- #highlight ⇒ Object
-
#initialize(lines, columns, y, x) ⇒ Window
constructor
A new instance of Window.
- #move(y, x) ⇒ Object
- #read_event ⇒ Object
- #read_event_nonblock ⇒ Object
- #recenter ⇒ Object
- #recenter_if_needed ⇒ Object
- #redisplay ⇒ Object
- #redraw ⇒ Object
- #resize(lines, columns) ⇒ Object
- #restore_point ⇒ Object
- #save_point ⇒ Object
- #scroll_down ⇒ Object
- #scroll_up ⇒ Object
- #shrink(n) ⇒ Object
- #shrink_if_larger_than_buffer ⇒ Object
- #split(other_lines = nil) ⇒ Object
- #wait_input(msecs) ⇒ Object
Constructor Details
#initialize(lines, columns, y, x) ⇒ Window
Returns a new instance of Window.
260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 |
# File 'lib/textbringer/window.rb', line 260 def initialize(lines, columns, y, x) @lines = lines @columns = columns @y = y @x = x initialize_window(lines, columns, y, x) @window.keypad = true @window.scrollok(false) @window.idlok(true) @buffer = nil @top_of_window = nil @bottom_of_window = nil @point_mark = nil @deleted = false @raw_key_buffer = [] @key_buffer = [] @cursor = Cursor.new(0, 0) @in_region = false @in_isearch = false @current_hl_face = nil end |
Instance Attribute Details
#bottom_of_window ⇒ Object (readonly)
Returns the value of attribute bottom_of_window.
258 259 260 |
# File 'lib/textbringer/window.rb', line 258 def bottom_of_window @bottom_of_window end |
#buffer ⇒ Object
Returns the value of attribute buffer.
257 258 259 |
# File 'lib/textbringer/window.rb', line 257 def buffer @buffer end |
#columns ⇒ Object (readonly)
Returns the value of attribute columns.
257 258 259 |
# File 'lib/textbringer/window.rb', line 257 def columns @columns end |
#cursor ⇒ Object (readonly)
Returns the value of attribute cursor.
258 259 260 |
# File 'lib/textbringer/window.rb', line 258 def cursor @cursor end |
#lines ⇒ Object (readonly)
Returns the value of attribute lines.
257 258 259 |
# File 'lib/textbringer/window.rb', line 257 def lines @lines end |
#mode_line ⇒ Object (readonly)
Returns the value of attribute mode_line.
257 258 259 |
# File 'lib/textbringer/window.rb', line 257 def mode_line @mode_line end |
#top_of_window ⇒ Object (readonly)
Returns the value of attribute top_of_window.
258 259 260 |
# File 'lib/textbringer/window.rb', line 258 def top_of_window @top_of_window end |
#window ⇒ Object (readonly)
Returns the value of attribute window.
257 258 259 |
# File 'lib/textbringer/window.rb', line 257 def window @window end |
#x ⇒ Object (readonly)
Returns the value of attribute x.
257 258 259 |
# File 'lib/textbringer/window.rb', line 257 def x @x end |
#y ⇒ Object (readonly)
Returns the value of attribute y.
257 258 259 |
# File 'lib/textbringer/window.rb', line 257 def y @y end |
Class Method Details
.beep ⇒ Object
253 254 255 |
# File 'lib/textbringer/window.rb', line 253 def self.beep Curses.beep end |
.colors ⇒ Object
131 132 133 |
# File 'lib/textbringer/window.rb', line 131 def self.colors Curses.colors end |
.columns ⇒ Object
226 227 228 |
# File 'lib/textbringer/window.rb', line 226 def self.columns Curses.cols end |
.current ⇒ Object Also known as: selected
45 46 47 |
# File 'lib/textbringer/window.rb', line 45 def self.current @@current end |
.current=(window) ⇒ Object
49 50 51 52 53 54 55 56 57 |
# File 'lib/textbringer/window.rb', line 49 def self.current=(window) if window.deleted? window = @@list.first end @@current.save_point if @@current && !@@current.deleted? @@current = window @@current.restore_point Buffer.current = window.buffer end |
.delete_other_windows ⇒ Object
94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 |
# File 'lib/textbringer/window.rb', line 94 def self.delete_other_windows if @@current.echo_area? raise EditorError, "Can't expand the echo area to full screen" end @@list.delete_if do |window| if window.current? || window.echo_area? false else window.delete true end end @@current.move(0, 0) @@current.resize(Window.lines - 1, @@current.columns) end |
.delete_window(target = @@current) ⇒ Object
71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 |
# File 'lib/textbringer/window.rb', line 71 def self.delete_window(target = @@current) if target.echo_area? raise EditorError, "Can't delete the echo area" end if @@list.size == 2 raise EditorError, "Can't delete the sole window" end i = @@list.index(target) return if i.nil? if i == 0 window = @@list[1] window.move(0, 0) else window = @@list[i - 1] end window.resize(target.lines + window.lines, window.columns) target.delete @@list.delete_at(i) if target == @@current self.current = window end end |
.echo_area ⇒ Object
119 120 121 |
# File 'lib/textbringer/window.rb', line 119 def self.echo_area @@echo_area end |
.has_colors=(value) ⇒ Object
123 124 125 |
# File 'lib/textbringer/window.rb', line 123 def self.has_colors=(value) @@has_colors = value end |
.has_colors? ⇒ Boolean
127 128 129 |
# File 'lib/textbringer/window.rb', line 127 def self.has_colors? @@has_colors end |
.lines ⇒ Object
222 223 224 |
# File 'lib/textbringer/window.rb', line 222 def self.lines Curses.lines end |
.list(include_echo_area: false) ⇒ Object
37 38 39 40 41 42 43 |
# File 'lib/textbringer/window.rb', line 37 def self.list(include_echo_area: false) if include_echo_area @@list.dup else @@list.reject(&:echo_area?) end end |
.load_faces ⇒ Object
140 141 142 143 144 |
# File 'lib/textbringer/window.rb', line 140 def self.load_faces require_relative "faces/basic" require_relative "faces/programming" require_relative "faces/completion" end |
.minibuffer_selected ⇒ Object
63 64 65 |
# File 'lib/textbringer/window.rb', line 63 def self.minibuffer_selected @@minibuffer_selected end |
.minibuffer_selected=(window) ⇒ Object
67 68 69 |
# File 'lib/textbringer/window.rb', line 67 def self.minibuffer_selected=(window) @@minibuffer_selected = window end |
.other_window ⇒ Object
110 111 112 113 114 115 116 117 |
# File 'lib/textbringer/window.rb', line 110 def self.other_window i = @@list.index(@@current) begin i += 1 window = @@list[i % @@list.size] end while !window.active? self.current = window end |
.redisplay ⇒ Object
191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 |
# File 'lib/textbringer/window.rb', line 191 def self.redisplay return if Controller.current.executing_keyboard_macro? return if Window.current.has_input? @@list.each do |window| window.redisplay unless window.current? end current.redisplay if defined?(FloatingWindow) # Render floating windows on top FloatingWindow.redisplay_all_floating # Ensure cursor position is from the current window, not floating windows # Refresh current window's cursor position after floating windows if !FloatingWindow.floating_windows.empty? current&.window&.noutrefresh end end update end |
.redraw ⇒ Object
210 211 212 213 214 215 216 |
# File 'lib/textbringer/window.rb', line 210 def self.redraw @@list.each do |window| window.redraw unless window.current? end current.redraw update end |
.resize ⇒ Object
230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 |
# File 'lib/textbringer/window.rb', line 230 def self.resize @@list.delete_if do |window| if !window.echo_area? && window.y > Window.lines - CONFIG[:window_min_height] window.delete true else false end end @@list.each_with_index do |window, i| unless window.echo_area? if i < @@list.size - 2 window.resize(window.lines, Window.columns) else window.resize(Window.lines - 1 - window.y, Window.columns) end end end @@echo_area.move(Window.lines - 1, 0) @@echo_area.resize(1, Window.columns) end |
.set_default_colors(fg, bg) ⇒ Object
135 136 137 138 |
# File 'lib/textbringer/window.rb', line 135 def self.set_default_colors(fg, bg) Curses.assume_default_colors(Color[fg], Color[bg]) Window.redraw end |
.start ⇒ Object
146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 |
# File 'lib/textbringer/window.rb', line 146 def self.start if @@started raise EditorError, "Already started" end Curses.init_screen Curses.noecho Curses.raw Curses.nonl self.has_colors = Curses.has_colors? if has_colors? Curses.start_color Curses.use_default_colors load_faces else Face.define :mode_line, reverse: true Face.define :region, reverse: true Face.define :isearch, underline: true end begin window = Textbringer::Window.new(Window.lines - 1, Window.columns, 0, 0) window.buffer = Buffer.new_buffer("*scratch*") @@list.push(window) Window.current = window @@echo_area = Textbringer::EchoArea.new(1, Window.columns, Window.lines - 1, 0) Buffer.minibuffer.keymap = MINIBUFFER_LOCAL_MAP @@echo_area.buffer = Buffer.minibuffer @@list.push(@@echo_area) @@started = true yield ensure Buffer.current&.input_method&.on_deactivate @@list.each do |win| win.close end @@list.clear Curses.echo Curses.noraw Curses.nl Curses.close_screen @@started = false end end |
.update ⇒ Object
218 219 220 |
# File 'lib/textbringer/window.rb', line 218 def self.update Curses.doupdate end |
Instance Method Details
#active? ⇒ Boolean
286 287 288 |
# File 'lib/textbringer/window.rb', line 286 def active? true end |
#close ⇒ Object
305 306 307 |
# File 'lib/textbringer/window.rb', line 305 def close @window.close end |
#current? ⇒ Boolean
335 336 337 |
# File 'lib/textbringer/window.rb', line 335 def current? self == @@current end |
#current_or_minibuffer_selected? ⇒ Boolean
339 340 341 |
# File 'lib/textbringer/window.rb', line 339 def current_or_minibuffer_selected? self == @@current || self == @@minibuffer_selected end |
#delete ⇒ Object
294 295 296 297 298 299 300 301 302 303 |
# File 'lib/textbringer/window.rb', line 294 def delete unless @deleted if current? Window.current = @@list.first end delete_marks @window.close @deleted = true end end |
#deleted? ⇒ Boolean
290 291 292 |
# File 'lib/textbringer/window.rb', line 290 def deleted? @deleted end |
#echo_area? ⇒ Boolean
282 283 284 |
# File 'lib/textbringer/window.rb', line 282 def echo_area? false end |
#enlarge(n) ⇒ Object
618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 |
# File 'lib/textbringer/window.rb', line 618 def enlarge(n) if n > 0 max_height = Window.lines - CONFIG[:window_min_height] * (@@list.size - 2) - 1 new_lines = [lines + n, max_height].min needed_lines = new_lines - lines resize(new_lines, columns) i = @@list.index(self) indices = (i + 1).upto(@@list.size - 2).to_a + (i - 1).downto(0).to_a indices.each do |j| break if needed_lines == 0 window = @@list[j] extended_lines = [ window.lines - CONFIG[:window_min_height], needed_lines ].min window.resize(window.lines - extended_lines, window.columns) needed_lines -= extended_lines end y = 0 @@list.each do |win| win.move(y, win.x) y += win.lines end elsif n < 0 && @@list.size > 2 new_lines = [lines + n, CONFIG[:window_min_height]].max diff = lines - new_lines resize(new_lines, columns) i = @@list.index(self) if i < @@list.size - 2 window = @@list[i + 1] window.move(window.y - diff, window.x) else window = @@list[i - 1] move(self.y + diff, self.x) end window.resize(window.lines + diff, window.columns) end end |
#has_input? ⇒ Boolean
386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 |
# File 'lib/textbringer/window.rb', line 386 def has_input? if !@raw_key_buffer.empty? || !@key_buffer.empty? return true end @window.nodelay = true begin c = @window.get_char if c @raw_key_buffer.push(c) end !c.nil? ensure @window.nodelay = false end end |
#highlight ⇒ Object
402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 |
# File 'lib/textbringer/window.rb', line 402 def highlight @highlight_on = {} @highlight_off = {} return if !@@has_colors || !CONFIG[:syntax_highlight] || @buffer.binary? syntax_table = @buffer.mode.syntax_table || DEFAULT_SYNTAX_TABLE if @buffer.bytesize < CONFIG[:highlight_buffer_size_limit] base_pos = @buffer.point_min s = @buffer.to_s else base_pos = @buffer.point len = columns * (lines - 1) / 2 * 3 s = @buffer.substring(@buffer.point, @buffer.point + len).scrub("") end return if !s.valid_encoding? re_str = syntax_table.map { |name, re| "(?<#{name}>#{re})" }.join("|") re = Regexp.new(re_str) names = syntax_table.keys s.scan(re) do b = base_pos + $`.bytesize e = b + $&.bytesize if b < @buffer.point && @buffer.point < e b = @buffer.point end name = names.find { |n| $~[n] } face = Face[name] if face @highlight_on[b] = face @highlight_off[e] = true end end end |
#move(y, x) ⇒ Object
543 544 545 546 547 548 |
# File 'lib/textbringer/window.rb', line 543 def move(y, x) @y = y @x = x @window.move(y, x) @mode_line.move(y + @window.maxy, x) end |
#read_event ⇒ Object
343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 |
# File 'lib/textbringer/window.rb', line 343 def read_event key = get_char if key.is_a?(Integer) if HAVE_GET_KEY_MODIFIERS if Curses::ALT_0 <= key && key <= Curses::ALT_9 @key_buffer.push((key - ALT_NUMBER_BASE).chr) return "\e" elsif Curses::ALT_A <= key && key <= Curses::ALT_Z @key_buffer.push((key - ALT_ALPHA_BASE).chr) return "\e" end end KEY_NAMES[key] || key else key&.encode(Encoding::UTF_8) end end |
#read_event_nonblock ⇒ Object
361 362 363 364 365 366 367 368 |
# File 'lib/textbringer/window.rb', line 361 def read_event_nonblock @window.nodelay = true begin read_event ensure @window.nodelay = false end end |
#recenter ⇒ Object
558 559 560 561 562 563 564 565 566 567 568 569 |
# File 'lib/textbringer/window.rb', line 558 def recenter @buffer.save_point do |saved| max = (lines - 1) / 2 count = beginning_of_line_and_count(max) while count < max break if @buffer.point == 0 @buffer.backward_char count += beginning_of_line_and_count(max - count - 1) + 1 end @buffer.mark_to_point(@top_of_window) end end |
#recenter_if_needed ⇒ Object
571 572 573 574 575 576 |
# File 'lib/textbringer/window.rb', line 571 def recenter_if_needed if @buffer.point_before_mark?(@top_of_window) || @buffer.point_after_mark?(@bottom_of_window) recenter end end |
#redisplay ⇒ Object
436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 |
# File 'lib/textbringer/window.rb', line 436 def redisplay return if @buffer.nil? redisplay_mode_line @buffer.save_point do |saved| if current? point = saved else point = @point_mark @buffer.point_to_mark(@point_mark) end framer @cursor.y = @cursor.x = 0 @buffer.point_to_mark(@top_of_window) highlight @window.erase @window.setpos(0, 0) @in_region = false @in_isearch = false @current_hl_face = nil if current_or_minibuffer_selected? && @buffer.visible_mark && @buffer.point_after_mark?(@buffer.visible_mark) @in_region = true end if current_or_minibuffer_selected? && @buffer.isearch_mark && @buffer.point_after_mark?(@buffer.isearch_mark) @in_region = false @in_isearch = true end apply_window_attrs(@window) while !@buffer.end_of_buffer? cury = @window.cury curx = @window.curx update_cursor_and_attr(point, cury, curx) if @highlight_off[@buffer.point] @current_hl_face = nil apply_window_attrs(@window) end if face = @highlight_on[@buffer.point] @current_hl_face = face apply_window_attrs(@window) end c = @buffer.char_after if c == "\n" # Fill to end of line with region background if in region if @in_region remaining = columns - curx @window.addstr(" " * remaining) if remaining > 0 else @window.clrtoeol end break if cury == lines - 2 # lines include mode line @window.setpos(cury + 1, 0) @buffer.forward_char next elsif c == "\t" n = calc_tab_width(curx) c = " " * n else c = compose_character(point, cury, curx, c) end s = escape(c) if curx < columns - 4 newx = nil else newx = curx + Buffer.display_width(s) if newx > columns if cury == lines - 2 break else # Fill to end of line with region background if in region if @in_region remaining = columns - curx @window.addstr(" " * remaining) if remaining > 0 else @window.clrtoeol end @window.setpos(cury + 1, 0) end end end @window.addstr(s) break if newx == columns && cury == lines - 2 @buffer.forward_char end apply_window_attrs(@window) @buffer.mark_to_point(@bottom_of_window) if @buffer.point_at_mark?(point) @cursor.y = @window.cury @cursor.x = @window.curx end if @cursor.x == columns - 1 c = @buffer.char_after(point.location) if c && Buffer.display_width(c) > 1 @cursor.y += 1 @cursor.x = 0 end end @window.setpos(@cursor.y, @cursor.x) @window.noutrefresh end end |
#redraw ⇒ Object
538 539 540 541 |
# File 'lib/textbringer/window.rb', line 538 def redraw @window.redraw @mode_line.redraw end |
#resize(lines, columns) ⇒ Object
550 551 552 553 554 555 556 |
# File 'lib/textbringer/window.rb', line 550 def resize(lines, columns) @lines = lines @columns = columns @window.resize(lines - 1, columns) @mode_line.move(@y + lines - 1, @x) @mode_line.resize(1, columns) end |
#restore_point ⇒ Object
331 332 333 |
# File 'lib/textbringer/window.rb', line 331 def restore_point @buffer.point_to_mark(@point_mark) end |
#save_point ⇒ Object
323 324 325 326 327 328 329 |
# File 'lib/textbringer/window.rb', line 323 def save_point @buffer[:top_of_window] ||= @buffer.new_mark @buffer[:top_of_window].location = @top_of_window.location @buffer[:bottom_of_window] ||= @buffer.new_mark @buffer[:bottom_of_window].location = @bottom_of_window.location @buffer.mark_to_point(@point_mark) end |
#scroll_down ⇒ Object
588 589 590 591 592 593 594 595 596 |
# File 'lib/textbringer/window.rb', line 588 def scroll_down if @top_of_window.location == @buffer.point_min raise RangeError, "Beginning of buffer" end @buffer.point_to_mark(@top_of_window) @buffer.next_line @buffer.beginning_of_line @top_of_window.location = 0 end |
#scroll_up ⇒ Object
578 579 580 581 582 583 584 585 586 |
# File 'lib/textbringer/window.rb', line 578 def scroll_up if @bottom_of_window.location == @buffer.point_max raise RangeError, "End of buffer" end @buffer.point_to_mark(@bottom_of_window) @buffer.previous_line @buffer.beginning_of_line @buffer.mark_to_point(@top_of_window) end |
#shrink(n) ⇒ Object
659 660 661 |
# File 'lib/textbringer/window.rb', line 659 def shrink(n) enlarge(-n) end |
#shrink_if_larger_than_buffer ⇒ Object
663 664 665 666 667 668 669 670 671 672 673 674 675 676 |
# File 'lib/textbringer/window.rb', line 663 def shrink_if_larger_than_buffer @buffer.save_point do @buffer.end_of_buffer @buffer.skip_re_backward(/\s/) count = beginning_of_line_and_count(Window.lines) + 1 while !@buffer.beginning_of_buffer? @buffer.backward_char count += beginning_of_line_and_count(Window.lines) + 1 end if lines - 1 > count shrink(lines - 1 - count) end end end |
#split(other_lines = nil) ⇒ Object
598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 |
# File 'lib/textbringer/window.rb', line 598 def split(other_lines = nil) old_lines = lines if other_lines if other_lines < CONFIG[:window_min_height] raise EditorError, "Window too small" end new_lines = lines - other_lines else new_lines = (old_lines / 2.0).ceil end if new_lines < CONFIG[:window_min_height] raise EditorError, "Window too small" end resize(new_lines, columns) new_window = Window.new(old_lines - new_lines, columns, y + new_lines, x) new_window.buffer = buffer i = @@list.index(self) @@list.insert(i + 1, new_window) end |
#wait_input(msecs) ⇒ Object
370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 |
# File 'lib/textbringer/window.rb', line 370 def wait_input(msecs) if !@raw_key_buffer.empty? || !@key_buffer.empty? return @raw_key_buffer.first || @key_buffer.first end @window.timeout = msecs begin c = @window.get_char if c @raw_key_buffer.push(c) end c ensure @window.timeout = -1 end end |